function res = set_partition(set,idxset,tag) %#ok<INUSD>
% Result = set_partition(Input-Data, Index-Set, Tag)
% A partitioning function (as required by the generic functions utl_crossval, utl_searchmodel, utl_nested_crossval) that takes a dataset
% (either epoched or continuous) and index set (either of the samples or of the epochs), and returns a reduced (subset) dataset.
% 
% In:
%   Input-Data : continuous or epoched input dataset
%
%   Index-Set  : either [] or a vector of indices; when [], the function returns the cardinality (maximum index) of the index set,
%                and otherwise, the function returns the dataset reduced those indices.
%
% Out:
%   Result : Reduced dataset or index set cardinality.
%
%                           Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                           2010-04-01


if ~exp_begindef('filter','memoize',0,'fingerprint_create','passthrough') return; end

if strcmp(idxset,'all')
    % in the online case we just return the set; this is an undocumented way to call the function; in the line prior to exp_enddef, the function
    % sets its online expression so that it is called with this argument by the online system (effectively disabling partitioning online).
    res = set;
elseif isempty(idxset)
    % in this case we compute the index set size;
    if ~isempty(set.epoch) || ndims(set.data) == 3
        % epoched dataset: return the number of epochs
        res = size(set.data,3);
    else
        % continuous dataset: return the number of samples
        res = size(set.data,2);
        if size(set.data,3) ~= 1
            error('dataset has no proper epoch field, but appears to be epoched nevertheless.'); end
    end
else
    % in this case we reindex the data
    if ~isempty(set.epoch)
        evalc('res = pop_select(set,''trial'',idxset,''sort'',0);');
    else
        % create a logical mask from the index set
        mask = false(1,size(set.data,2)); mask(idxset) = true;
        % find the inclusive, 1-based intervals of non-zeros
        intervals = reshape(find(diff([false mask false])),2,[])';
        intervals(:,2) = intervals(:,2)-1; %#ok<NASGU>
        evalc('res = pop_select(set,''point'',intervals,''sort'',0);');
    end
end

idxset = 'all'; %#ok<NASGU>; set up the online arguments
exp_enddef;
