function matches = exp_count(exp,match,lev)
% Matches = exp_count(In-Expression, Match-Expression, Level)
% counts the number of subexpressions matching Match-Expression in In-Expression (at the specified levels)
%
% In:
%   In-Expression    : some pure expression in which subexpressions are matched
%   Match-Expression : the expression that should be matched
%   Level            : level specification, currently only Inf is allowed, meaning all levels of the expression
%
% Out:
%   Matches          : number of matching subexpressions in In-Expression
%
%                                       Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                                       2010-04-26

if ~exp_begindef('symbolic') return; end

if ~exist('lev','var') || ~isequal(lev,Inf)
    error('currently, the only valid level is Inf, and it must be specified (for future compatibility)'); end

matches = utl_count(exp,match,lev);

exp_enddef;