function eeg = flt_selchans(eeg,channels)
% Result-Set = set_fltchans(Input-Set, Channels)
% Selects a subset of channels from the given dataset.
%
% Channel (or sensor) selection is a simple and effective method to constrain the complexity (and thus shorten computation time and/or improve robustness)
% of later stages in a paradigm. Sometimes, it is also employed to approximately restrict a paradigm to a sub-portion of brain processes (by using only channels
% directly above the brain region of interest), but this is not guaranteed to have the desired effect, since eletromagnetic signals emitted from any point in the 
% brain are practically captured by every sensor (due to volume conduction). Other uses of channel selection are to exclude bad channels in a faulty recording
% or to simulate the behavior of a paradigm running on a subset of the sensors (e.g., for cost reduction purposes).
% 
% In:
%   Input-Set : Data set set
%
%   Channels  : vector of channel indices or cell array of channel names
%
% Out:
%   Result-Set: EEGLAB data set restricted to the supplied channels (that were found in the original set)
%
%                               Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                               2010-04-17

if ~exp_begindef('filter') return; end

eeg = pop_select(eeg,'channel',set_chanid(eeg,channels),'sort',0);

exp_enddef;