function res = io_load(filename,varargin)
% function contents = io_load(Filename,Arguments...)
% Like MATLAB's builtin load(), except that the filename may be a platform-independent path.
%
% In:
%   Filename    :   Platform-independent file name.
%
%   Arguments...:   optional arguments to load, e.g. the list of variables, or load options
% 
% Out:
%   Contents    :   structure of contents of the file, if desired (if not specified, 
%                   variables are loaded into the workspace)
%
% Example:
%   io_loadmat('data:/myfile.mat');
%
% See also:
%   io_save()
%                                                     
%                               Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                               2010-06-21

contents = load(env_translatepath(filename),varargin{:});

if nargout > 0
    res = contents;
else
    for fn = fieldnames(contents)'
        assignin('caller',fn{1},contents.(fn{1})); end    
end