function varargout = cellfun_par(fun,varargin)
% parallel cellfun; same syntax & semantics as MATLAB's builtin cellfun()
% uses the parallelization settings specified in the global variable bcilab.parallel

% find out which parts of varargin are arguments to fun and which are options of cellfun()
cellmask = cellfun(@(x)iscell(x),varargin);
argsidxs = 1:find(~[cellmask false],1)-1;
optsidxs = argsidxs(end):length(varargin); %#ok<NASGU>
% create a batch of tasks
for i=1:numel(varargin{1})
    subargs = {};
    for k=argsidxs
        subargs{k} = varargin{k}{i}; end
    tasks{i} = {@hlp_wrapresults,fun,subargs{:}};
end
% and schedule it
results = reshape(hlp_schedule(tasks,'pool','global','engine','global','policy','global'),size(varargin{1}));
% check whether UniformOutput is enabled (default: yes)
uo = strcmp(varargin,'UniformOutput');
uo = ~(any(uo) && ~varargin{find(uo)+1});
% swap the outer and inner dimensions of results
for i=1:length(results{1})
    varargout{i} = cellfun(@(x)x(i),results); end
if uo
    % cat the results for each of the function's outputs
    for i=1:length(varargout)
        varargout{i} = reshape(cat(1,varargout{i}{:}), size(varargout{1})); end
end
