function [parameters,states] = bci_Construct
% [Parameters,States] = bci_Construct()
% Handle a BCI2000 Construct() event - declare parameters that should be managed by the system (e.g. available to the user for editing, saved/restored between sessions)
%
% Out:
%   Parameters : cell array of strings, which are BCI2000 parameter declaration lines 
%                (see http://www.bci2000.org/wiki/index.php/Technical_Reference:Parameter_Definition)
%
%   States : cell array of strings, which are BCI2000 state declaration lines
%            (see http://www.bci2000.org/wiki/index.php/Technical_Reference:State_Definition)
%
%                               Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                               2010-09-08



states = {};
parameters = {...
    'BCILAB string Biopredictor= predictor.mat // The biopredictor to be used for making predictions (inputfile)',...
    'BCILAB string MetaFile= metadata.set // An EEGLAB-supported file which specifies the meta-data of the strream (inputfile)', ...
    'BCILAB string OutputForm= expectation // can be ''expectation'', ''distribution'', or ''mode''. See README.txt in your BCILAB/online_plugins/BCI2000 directory.', ...
    'BCILAB int OutputRate= 10 10 0 1000 // Rate at which the predictor''s outputs will be sampled, in Hz', ...
    'BCILAB int OutputDims= 1 1 1 1000 // Tentative number of output stream channels; this should match the chosen predictor and output form.', ...
    'BCILAB float MaxCPU= 0.6 0.6 0 1 // Maximum fraction of CPU time allocated for this predictor', ...
    };
