function bci_Initialize(in_signal_dim, out_signal_dim)
% bci_Initialize(In-Signal-Dim,Out-Signal-Dim)
% Assuming that bci_Preflight completed successfully, set the global state variables in accordance with the bci_Parameters which have been assigned by the BCI2000 runtime
%
% In:
%  In-Signal-Dim  : a two-element vector of the form [#Channels, #Samples] which describes the input signal's dimensions
%
% Out:
%  Out-Signal-Dim : a two-element vector of the form [#Channels, #Samples] which describes the input signal's dimensions
%
%                               Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                               2010-09-08

% these variables are shared with Initialize and Process
global predictor meta_data in_samples out_samples next_outputs;

if isfield(meta_data,'channels')
    if isfield(meta_data,'chanlocs')
        [meta_data.chanlocs.labels] = deal(meta_data.channels{:}); 
    else
        meta_data.chanlocs = struct('labels',meta_data.channels);
    end
    meta_data = rmfield(meta_data,'channels');
end

% load a predictor
onl_loaddetector('BCILAB',predictor);

% set up the stream
onl_setup(meta_data.srate, meta_data.chanlocs, meta_data);

% set the consumed/computed sample numbers to zero
in_samples = 0;  % number of samples consumed & emitted so far
out_samples = 0; % number of samples computed so far (computation happens at a usually lower rate)

% this should be set in bci_StartRun
next_outputs = {zeros(length(meta_data.chanlocs),1)};
