function        [b1,b2] = vis_dplot(EEG,mrk_sel1,mrk_sel2,iva_sel,chn_sel,frq_sel)
% [ERP1,ERP2] = vis_dplot(EEG,marker_selection_class1,marker_selection_class2,interval_selection,channel_selection,frequency_selection)
%      
%   Plots Difference-ERDs in two classes selected by marker-sets. 
%
%
% IN:
%       EEG                     :       PhyPA-Datastructure, based
%                                       on EEGlabs .set-Structure.
%       marker_selection_class1 :     	String-Selection of eventmarkers to 
%                                       be substracted in the plot [as baseline].
%
%       marker_selection_class2 :     	String-Selection of eventmarkers to 
%                                       be added in the plot [as signal].
%
%       interval_selection      :       2-dim vector containing begin and
%                                       end in seconds for epoching the
%                                       data relative to the
%                                       marker_selections.
%       channel-selection       :       Numerical- or String-Array containing 
%                                       the chosen channels, e.g. [13:23] or 
%                                       {'C3','C4'}.
%       frequency-selection     :       2-dim vector containing high- and
%                                       lowpass frequency for
%                                       bandpassfilter, e.g. [0.1 17].
%
% OUT:
%       ERP1                    :       ERP of class selected by
%                                       marker_selection_class1
%
%       ERP2                    :       ERP of class selected by
%                                       marker_selection_class2
%
% See also: bci_preproc(), set_makepos()
%
%                                               Thorsten Zander, Team PhyPA,
%                                               FG MMS, TU-Berlin, 20.06.2007
%

%b1 = bci_preproc(EEG,@para_csp_train,'events',mrk_sel1,'window',iva_sel, 'filter',{'fft-nownd', frq_sel}, 'channels', chn_sel);
%b2 = bci_preproc(EEG,@para_csp_train,'events',mrk_sel2,'window',iva_sel, 'filter',{'fft-nownd', frq_sel}, 'channels', chn_sel);
b1 = bci_preproc(EEG,@para_csp_train,'events',mrk_sel1,'window',iva_sel, 'filter',{'fft-nownd', frq_sel}, 'channels', chn_sel);
b2 = bci_preproc(EEG,@para_csp_train,'events',mrk_sel2,'window',iva_sel,'filter',{'fft-nownd', frq_sel}, 'channels', chn_sel);

b  = b1;
b1 = squeeze(mean(b1.data,3));
b2 = squeeze(mean(b2.data,3));
figure;

for i=1:length(chn_sel)
    subplot(length(chn_sel),1,i);
    plot(b2(i,:)-b1(i,:));
    title(b.chanlocs(i).labels);
end
            
                        
                        
   