function matches = utl_cases(x,match,lev)
% Matches = utl_cases(In-Expression, Match-Expression, Level)
% finds all occurrences of Match-Expression in In-Expression (at the specified levels), and 
% aggregates them in the Matches cell array 
%
% In:
%   In-Expression    : some pure expression in which subexpressions are matched
%   Match-Expression : the expression that should be matched
%   Level            : level specification, currently only Inf is allowed, meaning that all levels of the expression are checked
%
% Out:
%   Matches          : cell array of all subexpressions that are matched by Match-Expression
%
%                                       Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                                       2010-04-25

if isfield(x,{'head','parts'})
    % canonical expression: first match everything
    if utl_match(x,match,struct())
        matches = {x};
    else
        matches = {};
    end
    % then recurse to sub-parts
    for i=1:length(x.parts)
        matches = [matches utl_cases(x.parts{i},match,lev)]; end
elseif isfield(x,'bcilab') && isfield(x.bcilab,'excpression')
    % impure expression: descend
    matches = utl_cases(x.bcilab.expression, match, lev);
elseif utl_match(x,match,struct())    
    % anything else...
    matches = {x};
else
    matches = {};
end

