function det = utl_complete_detector(det,func,data)
% internal utility: complete the definition of a detector, derived from the paradigm function and and the training dataset
%
%                               Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                               2010-04-28

% the .bcilab field holds the data that is of interest to the offline/online evaluation system
% it is allowed for the paradigm to fill in the following field manually, in order to yield custom behavior
if ~isfield(det,'bcilab')
    det.bcilab = struct(); end

% add the online expression, if necessary
if ~isfield(det.bcilab,'online_expression')
    det.bcilab.online_expression = data.bcilab.online_expression; end

% add the processing window length, if necessary
if ~isfield(det.bcilab,'window_length')
    % note: this will have to be extended for other data formats, such as the STUDY.
	det.bcilab.window_length = size(data.data,2); end

% add the prediction function, if necessary
if ~isfield(det.bcilab,'prediction_function')
    if isequal(1,strfind(char(func),'para_'))
        % the func is not directly the paradigm function (but some wrapper); look for it!
        try
            func(struct(),'prediction',struct());
        catch
            e = lasterror; %#ok<LERR>
            % the stack should in principle contain the paradigm name (except if the paradigm catches all its exceptions)
            found = 0;
            for i=1:length(e.stack)
                funcname = e.stack(i).name;
                if isequal(1,strfind(funcname,'para_'))
                    found = 1; break; end
            end
            if ~found
                error('cannot resolve the predictor function to include in the detector; please check this or annotate the detector.'); end
        end        
        func = eval(['@' funcname]);
    end
    det.bcilab.prediction_function = func;
end

% add the channel locations (these are technically redundant, as they can be derived from the preprocessing chain)
if ~isfield(det.bcilab,'channel_locations') && isfield(data,'chanlocs')
    det.bcilab.channel_locations = data.chanlocs; end
