function result = utl_evaluate_fold(opts,data,inds)
% core function of utl_crossval; learns a model on a subset of the data (indexed by inds{1}), 
% and tests its predictions on another subset of the data (indexed by inds{2})
%
% In:
%   Options : options struct as used for utl_crossval
%
%   Data    : a representation of some (indexable) data
%
%   Indices : a cell array of {training indices, test indices}
%
% Out:
%   Result  : a cell array of {real target values, predicted target values}
%
% See also:
%   utl_crossval
%
%                               Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                               2010-04-07

% learn a model
model = opts.trainer(opts.partitioner(data,inds{1},opts.args),opts.args{:});

% and record both the real test targets (result{1}) and the prediction on the test data (result{2})
testset = opts.partitioner(data,inds{2},model);
result = {opts.target(testset), opts.tester(testset,model)};
