function x = utl_purify_expression(x)
% Output-Expression = utl_purify_expression(Input-Expression)
% recursively remove impure elements from an expression (i.e. x.bcilab.expression is replaced by x).
% we assume that the head is not an impure expression...
%
%                                                     Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                                                     2010-05-23

if isfield(x,{'head','parts'})
    % x is a canonical expression: recurse
    for i=1:length(x.parts)
        x.parts{i} = utl_purify_expression(x.parts{i}); end
elseif isfield(x,'bcilab') && isfield(x.bcilab,'expression')
    % x is an impure expression: purify & recurse
    x = utl_purify_expression(x.bcilab.expression);
end