function r = utl_same(a,b)
% check whether two expressions are structurally identical; differs from isequal() in the following ways:
%  a) ignores the values associated with impure expressions
%  b) compares lambda expressions by their string representation; necessary for exp_symbol to work as expected
%  note: exp_symbol('x') and @x are considered non-equal by utl_same.
%
%                                       Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                                       2010-04-19

if isstruct(a) && isstruct(b)
    % structures may be impure expressions    
    if isfield(a,'bcilab') && isfield(a.bcilab,'expression')
        a = a.bcilab.expression; end
    if isfield(b,'bcilab') && isfield(b.bcilab,'expression')
        b = b.bcilab.expression; end
    
    % or canonically represented expressions
    if isfield(a,{'head','parts'}) & isfield(b,{'head','parts'}) %#ok<AND2>
        if ~isequal(a.head,b.head) || (length(a.parts) ~= length(b.parts))
            r = 0;
        else
            for k=1:length(a.parts)
                if ~utl_same(a.parts{k},b.parts{k})
                    r = 0;
                    return;
                end
            end
            r = 1;
        end
    else
        % or general structures
        r = isequalwithequalnans(a,b);
    end        
elseif isa(a,'function_handle') && isa(b,'function_handle')
	% function handles are compared via their string representation
    r = strcmp(char(a),char(b));
else
    % everything else is compared using isqual
    r = isequalwithequalnans(a,b);
end