function independentModulator = make_independent_modulators(icSpectralDecomposition, numberOfIms, keepIcDecompositionActivation, calculateActivations, downsamplingRate)

if nargin<2
    numberOfIms = 20;
end;

if nargin<3
    keepIcDecompositionActivation = true;
end;

if nargin<4
    calculateActivations = false;
end;

if nargin<5
    downsamplingRate = 1;
end;

combinedTemplateActivation = accumulate_ic_spectral_decomposition_activations(icSpectralDecomposition);

independentModulator = [];

[wts sph] = binica(combinedTemplateActivation(:,1:downsamplingRate:end), 'extended', round(numberOfIms / 2), 'pca', numberOfIms,   'stop' , 1e-7,'maxsteps' , 1000);

unmix = wts * sph;
mix = pinv(unmix);

independentModulator.wts = wts;
independentModulator.sph = sph;
independentModulator.mix = mix;
independentModulator.unmix = unmix;

% calculate and place correlations in the im structure, this is useful for viuslization of sub-spaces
[independentModulator.correlation p]= corrcoef(combinedTemplateActivation');
% remove non-significent correlations
independentModulator.correlation(p>0.01) = 0;
independentModulator.correlation = independentModulator.correlation - diag(diag(independentModulator.correlation));

if calculateActivations
   independentModulator.activation = (wts * sph) *  combinedTemplateActivation;
else
    independentModulator.activation = [];
end;

independentModulator.icSpectralDecomposition = icSpectralDecomposition;

% remove activity of icDeccompositions to save memory if requested.
if ~keepIcDecompositionActivation
    for i=1:length(independentModulator.icSpectralDecomposition)
        independentModulator.icSpectralDecomposition(i).activation = [];
        independentModulator.icSpectralDecomposition(i).badFrames = [];
    end;
end;
