function plot_independent_modulator(independentModulator, imNumber, icNumber, plotHistogram)
% plot_independent_modulator(independentModulator, imNumber, plotHistogram)


if nargin<2 || isempty(imNumber)
    imNumber = 1:size(independentModulator.mix,2);
end;

if nargin<3 || isempty(icNumber)
    icNumber = 1:length(independentModulator.icSpectralDecomposition);
end;

if nargin<4
    plotHistogram = true;
end;

% disable histogram if IM activations are not available and cannot be re-calculated.
if plotHistogram
    independentModulator = calculate_independent_modulators_activation(independentModulator);
end;

if isempty(independentModulator.activation)
     plotHistogram = false;
     fprintf('IM activations are not present and cannot be re-calculatd, histograms will not be plotted.\n');
end;


numberOfTemplatesPerIc = independentModulator.icSpectralDecomposition(1).numberOfSpctralTemplates;
numberOfFrequencies = independentModulator.icSpectralDecomposition(1).numberOfFrequencies;
numberOfICs = length(icNumber);
numberOfIms = length(imNumber);

frequencies = independentModulator.icSpectralDecomposition(1).frequencies;

combinedImWeightsForIc =  zeros(numberOfIms, numberOfICs, numberOfFrequencies);
for imId = 1:numberOfIms
    imWeights = independentModulator.mix(:,imNumber(imId));
    for icId =1:numberOfICs
        for icSpectralTemplateId = 1:numberOfTemplatesPerIc
            newWeights(1,:,:) =  (independentModulator.icSpectralDecomposition(icNumber(icId)).mix(:, icSpectralTemplateId) * imWeights(((icNumber(icId)-1) * numberOfTemplatesPerIc) + icSpectralTemplateId))';
            combinedImWeightsForIc(imId, icId,:) = combinedImWeightsForIc(imId,icId,:) + newWeights;
        end;
    end;
end;

%%


if plotHistogram
    numberOfRows = numberOfICs + 1;
else
    numberOfRows = numberOfICs;
end;

figure;

limits = [min(combinedImWeightsForIc(:)) max(combinedImWeightsForIc(:))] * 1.1;

for icId =1:numberOfICs
    icAxis = sbplot(numberOfRows, 1, icId, 'visible','off'); % last plot is for IM histogram
    
    % plot IC scalpmap
    icScalpmapAxis  = sbplot(1, 8, 1, 'ax', icAxis, 'visible','off');
    set_top_as_parent_axis(icScalpmapAxis, icAxis);
    topoplot(independentModulator.icSpectralDecomposition(icNumber(icId)).scalpMap, independentModulator.icSpectralDecomposition(icNumber(icId)).chanlocs,'plotrad',0.5);
    
    % plot IM spectral weight on IC
    icIMsAxis  = sbplot(1, 8, [2 8], 'ax', icAxis, 'visible','off');
    
    % set top of the axis similar to its parent
    set_top_as_parent_axis(icIMsAxis, icAxis);
    
    for imId = 1:numberOfIms
        icIMAxis  = sbplot(1, numberOfIms,imId, 'ax', icIMsAxis, 'visible','on');
              
        % place IM titles on the first row
        if icId == 1
            title(['IM ' num2str(imNumber(imId))], 'fontsize', 14);
        end;
        
        % set top of the axis similar to its parent
        set_top_as_parent_axis(icIMAxis, icIMsAxis);
        
        hold on;
        %plot(frequencies, squeeze(combinedImWeightsForIc(imId,icId,:)), 'linewidth',3);
        area(frequencies, squeeze(combinedImWeightsForIc(imId,icId,:)));
        %plot(frequencies, squeeze(combinedImWeightsForIc(imId,icId,:)), '.k');
        hold off;
        
        xlim([min(frequencies) max(frequencies)]);
        set(gca, 'xscale','log');
        ylim(limits);
        
        % place x ticks only at the bottom
        if icId == numberOfICs
            set(gca, 'xtick',[min(frequencies) 5 7 10 17 30 50 100], 'fontsize',8);
        else
            set(gca, 'xtick',[]);
        end;
        
        % place y ticks only on the left side
        if imId ~= 1
            set(gca, 'ytick',[]);
        end;
        
        
        
    end;
    
end;

if plotHistogram
    imHistogramsAxis = sbplot(numberOfRows, 1, numberOfRows, 'visible','off'); % last plot is for IM histogram
    axisPosition = get(imHistogramsAxis, 'position');
    axisPosition(2) = axisPosition(2) - 0.02;
    set(imHistogramsAxis, 'position', axisPosition);
    
        for imId = 1:numberOfIms
        imHistogramAxis  = sbplot(1, numberOfIms,imId, 'ax', icIMsAxis, 'visible','on');
        set_top_as_parent_axis(imHistogramAxis, imHistogramsAxis);
        
        [n x] = hist(independentModulator.activation(imNumber(imId),:), 50);
        barHandle = bar(x, n, 1); % make bars touch each other
        set(barHandle, 'facecolor', [ 0 0 0])
        % set top of the axis similar to its parent
        
         set(imHistogramAxis, 'ytick',[]);
         set(imHistogramAxis, 'ylim',[0 max(n) * 1.1]);
         
         % plot a vertical line at x = 0
         line([0 0], [min(get(imHistogramAxis,'ylim')) max(get(imHistogramAxis,'ylim'))], 'color', [0.5 0.5 0.5]);
        end;
end;
