%% --- load raw dataset and set up data-specific parameters (here: eventtype->class mapping) ---
eeg = exp_eval(io_loadset('data:/christian/Demo/scp/VP004/Ch-noswitch.vhdr'));
channels = {'C3' 'Cz' 'C4' 'CP3' 'CPz' 'CP4' 'FC3' 'FCz' 'FC4'};

%% --- Self-Paced classification (hard!) ---
% first clean up the events for this... specifically, we forbid key presses which are either too early or too late
too_late = 5;     % both are relative to the beginning of the enter sequence (and therefore depend on its length)
too_early = 1.45; 
erased = [];
for m=1:length(eeg.event)
    % look for key presses...
    if strcmp(eeg.event(m).type,'S  4') || strcmp(eeg.event(m).type,'S  8')
        % following the self-paced indicator...
        if strcmp(eeg.event(m-1).type,'S 16')
            lat = (eeg.event(m).latency - eeg.event(m-1).latency) / eeg.srate;
            % which are illegal...
            if lat < too_early || lat > too_late
                erased = [erased m];
            end
        elseif ~strcmp(eeg.event(m).type,'S 18')
            % otherwise the key is illegal (note: switches are explicitly allowed)
            erased = [erased m];
        end
    end    
end
% ... and erase them
eeg.event = eeg.event(setdiff(1:length(eeg.event),erased));

% now inject jittered training sections relative to the key presses
% we want S5 for no-move left, S6 for move-left, S9 for no-move right and S10 for move right
key_markers = {'S  4', 'S  8'};
selfp_markers = {{'S  5', 'S  6'}, {'S  9', 'S 10'}}; 
nomov_markers = {'S  5', 'S  9'};
mov_markers = {'S  6', 'S 10'};
selfp_enter = 'S 16';

% training window parameters
nomov_begin = 1;    % beginning offset to the 'no-move' section, in ms (to take account for the irrelevant enter sequence)
nomov_end = -0.50;  % ending of the 'no-move' section, in ms relative to keypress (tradeoff between conservativity and coverage)
mov_begin = -0.125; % beginning of the 'move' section, in ms relative to keypress
mov_end = -0.1;     % end of the 'move' section, in ms releative to keypress
mov_safety = 0.5;   % the length of the safety margin after a 'move' section, which ensures that we don't treat any move data as nomove
prestim_time = -1;  % duration of the pre-stimulus phase, optionally to be taken as wait state
nomov_jitter_cnt = 7;  % number of jittered markers in the nomov section
mov_jitter_cnt = 7;    % number of jittered markers in the mov section

% inject jittered markers for left, right
for side = 1:2
    eeg = set_inject_events(eeg,'segment',{selfp_enter nomov_begin nomov_end key_markers{side}}, 'event', nomov_markers{side}, 'count',nomov_jitter_cnt);
    eeg = set_inject_events(eeg,'segment',{mov_begin mov_end key_markers{side}}, 'event', mov_markers{side}, 'count',mov_jitter_cnt);
end


%% train a model
windows = {[-1.6 -0.5; -1.2 -0.5; -0.5 0.45; -0.2 -0.175; -0.025 0], [-1.6 -0.5; -1.2 -0.5; -0.45 0.4; -0.2 -0.175; -0.025 0], [-1.6 -0.5; -1.2 -0.5; -0.9 -0.4; -0.45 0.4; -0.2 -0.175; -0.025 0]};
filters = {{'iir-lowpass',[2.5 14]}, {'iir-lowpass',[3 10]}, {'iir-lowpass',[4 16]}};
clear mcr mdl stats
for side = 1:2
    % common options for all paradigms
    opts = {'filter',filters{1}, 'wnds',windows{1}, 'channels',1:32, 'events',selfp_markers{side}, 'window',[-2 0]};
    
    % DAL-SCP
	[mcr{side},mdl{side},stats{side}] = bci_train({'data',eeg,'func',@para_dal_lf},opts{:})
    % channel-restricted LDA
    %[mcr{side},mdl{side},stats{side}] = bci_train({'data',eeg,'func',@para_windowmeans},opts{:}, 'channels',channels)
    % logreg
    %[mcr{side},mdl{side},stats{side}] = bci_train({'data',eeg,'func',@para_windowmeans},opts{:}, 'learner',{'logreg',1,'variant','l2'})
end


% %% optionally visualize the self-paced part of the data
% eeg = exp_eval(eeg);
% for side = 1:2
%     figure; vis_erp(bci_preproc(eeg,@para_windowmeans,'window',[-2 2],'events',selfp_markers{side},'filter',{'iir-lowpass',[2.5 14]},'channels',channels,'laplace','full-lap'),{ 'FC3' 'FCz' 'FC4';'C3' 'Cz' 'C4';'CP3' 'CPz' 'CP4'}');
% end



%% % --- Cued classification (easier!) ---
% % first, rename the S16 & S17 markers according to the subsequent key-presses... (into S20|21 for S16 and S22|23 for S17)
% % also, reject artifact markers (i.e. those for which the S16 marker came too early or the S17 marker came too late)
% too_late = 5;     % both are relative to the beginning of the enter sequence (and therefore depend on its length)
% too_early = 1.45; 
% erased = [];
% for m=2:length(eeg.event)
%     % we have an interesting keypress
%     if strcmp(eeg.event(m).type,'S  4') || strcmp(eeg.event(m).type,'S  8')
%         if strcmp(eeg.event(m-1).type,'S 16')
%             % this is supposedly a self-paced key marker, check if we are neither too late nor too early
%             lat = (eeg.event(m).latency - eeg.event(m-1).latency) / eeg.srate;
%             if lat >= too_early && lat <= too_late
%                 if strcmp(eeg.event(m).type,'S  4')
%                     eeg.event(m-1).type = 'S 20';
%                 else
%                     eeg.event(m-1).type = 'S 21';
%                 end
%             end
%             % erase these key marker in every case (since these are of no interest to us here)
%             erased = [erased m];
%         elseif strcmp(eeg.event(m-1).type,'S 17')
%             % this is supposedly a cued key marker, check if we are not too late
%             lat = (eeg.event(m).latency - eeg.event(m-1).latency) / eeg.srate;
%             if lat < too_early % consequently early enough for a cued keypress
%                 if strcmp(eeg.event(m).type,'S  4')
%                     eeg.event(m-1).type = 'S 22';
%                 else
%                     eeg.event(m-1).type = 'S 23';
%                 end
%             else
%                 % erase cued key markers which were too late
%                 erased = [erased m];
%             end
%         else
%             % something unexpected happened in between: erase
%             erased = [erased m];
%         end
%     end
% end
% % erase them
% eeg.event = eeg.event(setdiff(1:length(eeg.event),erased));
% 
% % insert jittered training markers
% key_markers = {'S  4', 'S  8'};
% train_markers = {{'S  5', 'S  6'}, {'S  9', 'S 10'}}; 
% nomov_markers = {'S  5', 'S  9'};
% mov_markers = {'S  6', 'S 10'};
% selfp_enter = 'S 16';
% % training window parameters
% nomov_begin = 1;    % beginning offset to the 'no-move' section, in ms
% nomov_end = 1.45;   % ending of the 'no-move' section, in ms
% mov_begin = -0.145; % beginning of the 'move' section, in ms relative to keypress
% mov_end = -0.1;     % end of the 'move' section, in ms releative to keypress
% nomov_jitter_cnt = 7; % number of jittered markers in the nomov section
% mov_jitter_cnt = 7;   % number of jittered markers in the mov section
% 
% % inject jittered markers for left, right
% for side = 1:2
%     eeg = set_inject_events(eeg,'segment',{'S 20' nomov_begin nomov_end}, 'count',nomov_jitter_cnt, 'event',nomov_markers{side});
%     eeg = set_inject_events(eeg,'segment',{'S 21' nomov_begin nomov_end}, 'count',nomov_jitter_cnt, 'event',nomov_markers{side});
%     eeg = set_inject_events(eeg,'segment',{mov_begin mov_end key_markers{side}}, 'count',mov_jitter_cnt, 'event',mov_markers{side});
% end
% cue_markers = {{'S  5', 'S  6'}, {'S  9', 'S 10'}}; 
% %cue_markers = {{'S 20' 'S 22' 'safety'}, {'S 21' 'S 23' 'safety'}};     % note: this is the version without jittered training

% 
% %train a model!
% windows = {[-0.8 -0.5; -0.7 -0.4; -0.5 0.45; -0.2 -0.175; -0.025 0], [-0.7 -0.4; -0.5 0.45; -0.3 -0.275; -0.2 -0.175; -0.025 0]};
% % windows = {[0.3 0.6; 0.775 0.8; 0.875 0.9 ; 0.975 1],[0.775 0.8; 0.75 0.775]}; % note: this is the version without jittered training
% clear mcr mdl stats
% % for side = 1:2
% % 	[mcr{side},mdl{side},stats{side}] = bci_train({'data',eeg,'func',@para_windowmeans}, 'events',cue_markers{side}, 'window',[-2 0], ...
% %         'filter', {'iir-lowpass',[2.5 10]}, 'channels', channels, 'wnds',windows{1}, 'learner',{'logreg' 1 'variant' 'l2'});
% % end
% 
% for side = 1:2    
% 	[mcr{side},mdl{side},stats{side}] = bci_train({'data',eeg,'func',@para_dal_lf}, 'events',cue_markers{side}, 'window',[-2 0], ...
%         'srate',30, 'filter', {'iir-lowpass',[2.5 10]}, 'channels', 1:32, 'wnds',windows{1}, 'learner',{'dal' 1 'shape' [32,NaN]});
% end
% 














% 
% % --- train a model and obtain a default (cross-validated) performance estimate (here: mis-classification rate) as well as additional statistics ---
% 
% % select from a few different BCI approaches
% [mcr,mdl,stats] = bci_train({'data',raw,'func',@para_csp},'events',mrks);
% %[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_csp},'events',mrks,'learner',{'logreg' 1 'variant' 'l1'});
% %[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_csp},'events',mrks,'learner',{'dal' 1 'regularizer' 'l1'});
% %[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_csp},'events',mrks,'learner',{'logreg' search(2.^(-6:1:10)) 'variant' 'l1'});
% %[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_csp},'events',mrks,'window',{search(0:0.5:2) search(2.1:0.5:4.1)});
% %[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_csp},'events',mrks,'patterns',search(1,2,3));
% %[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_csp,'eval_scheme',{'chron',20,5}},'events',mrks,'window',[-0.5 2.5]);
% %[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_csp},'events',mrks,'learner',{'svmlinear'});
% %[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_csp},'events',mrks,'learner',{'svmlinear' 1 'variant','matlab'});
% 
%   disp(['training mis-classification rate: ' num2str(mcr*100,3) '%']);
% 
% 
% 
% % --- apply the model to the (here: same) raw dataset (pretending that this were some valid test session) ---
% % [pred,mcr2,stats2,targ] = bci_predict(mdl,raw);
% % 
% %   disp(['predicted classes: ',num2str(round(pred{2}*pred{3})')]);  % class probabilities * class values
% %   disp(['true classes     : ',num2str(round(targ)')]);
% 
% 
% 
% % --- for demo purposes, stream the raw dataset through the online system (predicting 3.5 seconds after each marker) ---
% % [opred,olat] = onl_stream(raw, mdl, 'mrks',mrks, 'offset',3.5*200);
% 
% %   disp(['online prediction; ',num2str(round(opred{2}*opred{3})')]);

