% --- load raw dataset and set up data-specific parameters (here: eventtype->class mapping) ---
raw = io_loadset('data:/christian/Demo/imag.vhdr');
mrks = {'S  1','S  2','S  3'};   % these are our class-indicating stimulus markers

raw = exp_eval(raw);
% add a third class to the data (the Rest condition...); for this, we add a marker 3.5s after each stimulus (this is when the rest condition begins...)
oldevents = raw.event;
for i=1:length(oldevents)
    if any(strcmp(oldevents(i).type,mrks) & [1 1 0])
        raw.event(end+1) = oldevents(i);
        raw.event(end).latency = raw.event(end).latency+3.5*raw.srate;
        raw.event(end).type = mrks{3};
    end
end
[x,inds] = sort([raw.event.latency]);
raw.event = raw.event(inds);

% --- train a model and obtain a default (cross-validated) performance estimate (here: mis-classification rate) as well as additional statistics ---

% select from a few different BCI approaches
%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_bandpower},'events',mrks,'window',[0.5 3.5],'learner',{'lda'});
%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_bandpower},'events',mrks,'window',[0.5 3.5],'learner',{'logreg' 1 'variant' 'vb'});
%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_bandpower},'events',mrks,'window',[0.5 3.5],'learner',{'logreg'});
%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_bandpower},'events',mrks,'window',[0.5 3.5],'learner',{'rvm'});
%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_bandpower},'events',mrks,'window',[0.5 3.5],'learner',{'lda', [], 'weight_bias',1, 'weight_cov',1});
%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_csp},'events',mrks,'window',[0.5 3.5],'learner',{'svmlight'});
%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_bandpower},'events',mrks,'window',[0.5 3.5],'learner',{'svmlight'});
%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_bandpower},'events',mrks,'window',[0.5 3.5],'learner',{'svmlinear' 1 'variant' 'matlab'});
%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_bandpower},'events',mrks,'window',[0.5 3.5],'learner',{'svmlinear' 1 'variant' 'primal'});
%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_bandpower},'events',mrks,'window',[0.5 3.5],'learner',{'svmperf'});
%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_bandpower},'events',mrks,'window',[0.5 3.5],'learner',{'logreg' 1 'variant' 'l1'});
[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_bandpower},'events',mrks,'window',[0.5 3.5],'learner',{'dal' 1 'regularizer' 'l1'});

%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_csp},'events',mrks);
%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_csp},'events',mrks,'learner',{'logreg' 1 'variant' 'l1'});
%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_csp},'events',mrks,'learner',{'logreg' search(2.^(-6:1:10)) 'variant' 'l1'});
%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_csp},'events',mrks,'window',{search(0:0.5:2) search(2.1:0.5:4.1)});
%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_csp},'events',mrks,'patterns',search(1,2,3));
%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_csp,'eval_scheme',{'chron',20,5}},'events',mrks,'window',[-0.5 2.5]);
%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_csp},'events',mrks,'learner',{'svmlinear'});
%[mcr,mdl,stats] = bci_train({'data',raw,'func',@para_csp},'events',mrks,'learner',{'svmlinear' 1 'variant','matlab'});

  disp(['training mis-classification rate: ' num2str(mcr*100,3) '%']);



% --- apply the model to the (here: same) raw dataset (pretending that this were some valid test session) ---
[pred,mcr2,stats2,targ] = bci_predict(mdl,raw);
% 
disp(['predicted classes: ',num2str(round(pred{2}*pred{3})')]);  % class probabilities * class values
disp(['true classes     : ',num2str(round(targ)')]);



% --- for demo purposes, stream the raw dataset through the online system (predicting 3.5 seconds after each marker) ---
% [opred,olat] = onl_stream(raw, mdl, 'mrks',mrks, 'offset',3.5*200);

%   disp(['online prediction; ',num2str(round(opred{2}*opred{3})')]);