function eeg = flt_attach_LLt(eeg, modelpath, smoothlen)
% attaches the per-model log-likelihood as new channels in the dataset

if ~exp_begindef('filter') return; end
if ~exist('smoothlen','var') smoothlen = 1; end

% write the data
floatwrite(eeg.data,[modelpath '/tempdata.fdt']);

% run amica to get LLt's
system(['/data/common/amica/amica ' fullfile(modelpath,'input.param')]);

% load LLt's and filter them 
m = loadmodout(modelpath);
smoothwnd = hann(smoothlen*eeg.srate); smoothwnd = smoothwnd/sum(smoothwnd);
LLt = filtfilt_fast(smoothwnd,1,m.LLt')';

% map to probabilities
v = zeros(size(LLt));
for k = 1:size(v,1)
    v(k,:) = 1./sum(exp(bsxfun(@minus,LLt,LLt(k,:))),1); end

% attach to dataset
eeg.data = [eeg.data; v];
eeg.nbchan = size(eeg.data,1);
channames = {};
for k=1:m.num_models
    channames{end+1} = ['prob' num2str(k)]; end
[eeg.chanlocs(end+1 : end+m.num_models)] = channames{:};

exp_enddef;
