function result = concerto_load_raw_reduced(session)
if ~exp_begindef('filter') return; end

session_files = {'data:/concerto/tim-session1/tim-session1-repaired-properchans.set', ... % special filenames for the first 2 (broken) sessions
                 'data:/concerto/tim-session2/tim-session2-repaired-properchans.set'};
 
iv = {'A','B','C','F','H'};          % names of the individual intervals
if session <= 4
    tone_track_names = [11 12 13 16 18]; % before session 5 we had different track names
else
    tone_track_names = [31 32 33 36 38]; % corresponding track names for in the control channel
end

margin = 5;  % margin between events in first and second half of each interval in seconds
mrate = 2;   % markers per second
srate = 512; % sampling rate

% load data & find interval ranges
if session > 2
    % load raw data...
    raw = exp_eval(io_loadset(['data:/concerto/tim-session' num2str(session) '/tim-session' num2str(session) '.bdf']));    
    % find the control channels...
    for c = 128:raw.nbchan
        channel = raw.data(c,:);
        values = unique(channel);
        if length(values) < 20 && ismember(20,values) && nnz(channel==0)/length(channel) > 0.95
            % less than 20 distinct values, contains 20, mostly zeros: we have found the Wii track
            wii_track = channel;
        elseif length(values) < 50 && length(intersect(values,tone_track_names)) == length(tone_track_names)
            % less than 100 distinct values & contains all tone tracks: we have found the tone track...
            tone_track = channel;
        end
    end
    % find the tone types per button press
    btn_types = tone_track(wii_track==20);
    % and the corresponding latencies
    btn_latencies = find(wii_track==20)/raw.srate;
    % now let's look for the ranges of the respective intervals...
    for i = 1:length(tone_track_names)
        % all candidate events to delimit the tone range
        candidates = find(btn_types==tone_track_names(i));
        % we take the last two of them (since a tone might have been repeated due to a recording breakdown)
        ranges(i,:) = btn_latencies(candidates([end-1 end]));        
    end
else
    % The first two sessions contain broken data and require special handling...
    raw = exp_eval(flt_resample(io_loadset(session_files{session}),[],srate));
    % we use the event channel to infer the button press ranges for the corresponding intervals
    ranges = find(strcmp('20',{raw.event.type}));
    if session == 1
        ranges = ranges([1 2; 3 4; 5 6; 11 12; 15 16]);
    else
        ranges = ranges([1 2; 3 4; 5 6; 11 11; 14 15]);
    end
    ranges = reshape([raw.event(ranges).latency]/raw.srate,size(ranges));
end

disp('ranges: ');
ranges(:,2)-ranges(:,1)

% add markers for each interval
result = raw;
for i=1:size(ranges,1)
    % time range of the interval
    range = ranges(i,:);
    if range(1) ~= range(2)    
        % insert first-half events
        result = exp_eval(set_inject_events(result,'event',[iv{i} '_h1'], 'segment',{range(1), mean(range)-margin/2}, 'count',mrate, 'counting','persecond', 'placement','equidistant'));
        % insert second-half events
        result = exp_eval(set_inject_events(result,'event',[iv{i} '_h2'], 'segment',{mean(range)+margin/2 range(2)}, 'count',mrate, 'counting','persecond', 'placement','equidistant'));
    end
end

% set up the proper channel names...
result.chanlocs = pop_readlocs(env_translatepath('data:/concerto/yellow-cap.elp'));
for i=137:result.nbchan
    result.chanlocs(i).labels = ['ctrl_' num2str(i)]; end
if session > 2
    result = pop_select(result,'channel',{result.chanlocs(1:136).labels}); end

exp_enddef('add_online',@rawdata);