function [eeg_cued, eeg_selfp] = do_chicken_train_online(origeeg, visualize, filename)
% train online classifiers for the chicken demo and store them locally in classifiers_chicken_curr.mat
% if the filename is 'geteeg', only the annotated data sets are returned
if ~exist('visualize') || isempty(visualize) visualize = 0; end
if ~exist('filename') filename = 'classifiers_chicken_curr'; end

origeeg = prc_unpack(origeeg);
origeeg.cache_term = utl_manglename('do_chicken_train_online_v.2',origeeg);
env_cachemode('bypass');

%channels = {'C3' 'Cz' 'C4' 'FCz' 'CP5' 'CP6' 'FC5' 'FC6'};
%channels = {'C3' 'Cz' 'C4' 'FCz' 'CP3' 'CP4' 'FC3' 'FC4'};
channels=1:32;


% --- 1. train the self-paced classifiers ---
eeg = origeeg;
disp('training self-paced classifier...');

% first clean up the events for this... specifically, we forbid key presses which are either too early or too late
too_late = 5;     % both are relative to the beginning of the enter sequence (and therefore depend on its length)
too_early = 1.45; 
erased = [];
for m=1:length(eeg.event)
    % look for key presses...
    if strcmp(eeg.event(m).type,'4') || strcmp(eeg.event(m).type,'8')
        % following the self-paced indicator...
        if strcmp(eeg.event(m-1).type,'16')
            lat = (eeg.event(m).latency - eeg.event(m-1).latency) / eeg.srate;
            % which are illegal...
            if lat < too_early || lat > too_late
                erased = [erased m];
            end
        elseif ~strcmp(eeg.event(m).type,'18')
            % otherwise the key is illegal (note: switches are explicitly allowed)
            erased = [erased m];
        end
    end    
end
% ... and erase them
eeg.event = eeg.event(setdiff(1:length(eeg.event),erased));


% now inject jittered training sections relative to the key presses
% we want S5 for no-move left, S6 for move-left, S9 for no-move right and S10 for move right
disp('  injecting jittered marker sections...');
key_markers = {'S  4', 'S  8'};
selfp_markers = {{'S  5', 'S  6', 'safety'}, {'S  9', 'S 10', 'safety'}}; 
nomov_markers = {'S  5', 'S  9'};
mov_markers = {'S  6', 'S 10'};
selfp_enter = 'S 16'; % basically the stimulus...

% training window parameters
nomov_begin = 1;    % beginning offset to the 'no-move' section, in ms (to take account for the irrelevant enter sequence)
nomov_end = -0.50;  % ending of the 'no-move' section, in ms relative to keypress (tradeoff between conservativity and coverage)
mov_begin = -0.125; % beginning of the 'move' section, in ms relative to keypress
mov_end = -0.1;     % end of the 'move' section, in ms releative to keypress
mov_safety = 0.5;   % the length of the safety margin after a 'move' section, which ensures that we don't treat any move data as nomove
prestim_time = -1;  % duration of the pre-stimulus phase, optionally to be taken as wait state
nomov_jitter_cnt = 7;  % number of jittered markers in the nomov section
mov_jitter_cnt = 7;    % number of jittered markers in the mov section

% inject jittered markers for left, right
for side = 1:2
    eeg = set_inject_jitters(eeg,{selfp_enter nomov_begin nomov_end key_markers{side}}, {nomov_jitter_cnt, {nomov_markers{side}}, 'random', 'total'});
    eeg = set_inject_jitters(eeg,{mov_begin mov_end key_markers{side}}, {mov_jitter_cnt, {mov_markers{side}}, 'random', 'total'});
end

% optionally visualize the self-paced part of the data
if visualize    
    for side = 1:2
        figure; vis_erp(bci_preproc(eeg,@para_simple_train,'window',[-2 2],'events',selfp_markers{side},'filter',{'noncausal-fft',[0.5 5]},'channels',channels,'laplace','full-lap'),{ 'FC3' 'FCz' 'FC4';'C3' 'Cz' 'C4';'CP3' 'CPz' 'CP4'}');
    end
    waitforbuttonpress;
end

% --- big experiment: self-paced lateralized classifiers ---
windows = {[-6 -0.5; -1.6 -0.5; -1.2 -0.5; -0.5 0.45; -0.2 -0.175; -0.025 0], [-1.6 -0.5; -1.2 -0.5; -0.45 0.4; -0.2 -0.175; -0.025 0], [-1.6 -0.5; -1.2 -0.5; -0.9 -0.4; -0.45 0.4; -0.2 -0.175; -0.025 0]};
classifiers = {hlp_search('lda-full',10), hlp_search('svm-poly',35)};
filters = {{'iir-lowpass',[2.5 14]}, {'iir-lowpass',[3 10]}, {'iir-lowpass',[4 16]}};
f=1; w=1;

% Thorsten, es ist genau das, was hier auskommentiert ist:
% 
% procX = bci_preproc(eeg,@para_simple_train,'srate',200,'laplace','no-lap','events',{'S  6' 'S 10' 'safety'},'window',[-2 0], 'filter',filters{f},'channels',channels);
% [err,classifierLR,x,statsX] = bci_train(@para_simple_train,procX,{'chron' 8 8},{'chron' 8 8},[],{windows{2}},{{'lda' 0.015}}); statsX
% % % % 
% % experimentally rename the S9 into S5 and the S10 into S6, and then try to train on unilateral movement onset.
% for m=1:length(eeg.event)
%     if strcmp(eeg.event(m).name,'S  9')
%         eeg.event(m).name = {'S  5'};
%         eeg.event(m).type = '5';
%     end
%     if strcmp(eeg.event(m).name,'S 10')
%         eeg.event(m).name = {'S  6'};
%         eeg.event(m).type = '6';
%     end
% end
% 
% procX = bci_preproc(eeg,@para_simple_train,'srate',200,'laplace','no-lap','events',{'S  5' 'S  6' 'safety'},'window',[-2 0], 'filter',filters{f},'channels',channels);
% [err,classifierNM,x,statsX] = bci_train(@para_simple_train,procX,{'chron' 8 8},{'chron' 8 8},[],{windows{2}},{{'lda' 0.015}}); statsX




% % preprocess & train for left, right
% if ~strcmp(filename,'geteeg')
%     disp('  training classifiers...');
%     windows = {[-6 -0.5; -1.6 -0.5; -1.2 -0.5; -0.5 0.45; -0.2 -0.175; -0.025 0], [-1.6 -0.5; -1.2 -0.5; -0.45 0.4; -0.2 -0.175; -0.025 0], [-1.6 -0.5; -1.2 -0.5; -0.9 -0.4; -0.45 0.4; -0.2 -0.175; -0.025 0]};
%     classifiers = {hlp_search('lda-full',10), hlp_search('svm-poly',35)};
%     filters = {{'iir-lowpass',[2.5 14]}, {'iir-lowpass',[3 10]}, {'iir-lowpass',[4 16]}};
%     f=1; w=1;
% %    for f=1:length(filters)
%  %       for w=1:length(windows)
%             for side = 1:2
%                 proc{side} = bci_preproc(eeg,@para_simple_train,'srate',200,'laplace','no-lap','events',selfp_markers{side},'window',[-6 0], 'filter',filters{f},'channels',channels);
%                 [err,classifier{side},x,stats{side}] = bci_train(@para_simple_train,proc{side},{'chron' 8 8},{'chron' 8 8},[],{windows{w}},{{'lda' 0.015}}); stats{side}
%             end
% %        end
%  %   end
%     % and remember the classifiers
%     cla_sl = classifier{1};
%     cla_sr = classifier{2};
% end
% eeg_selfp = eeg;

% --- 2. train the cue-based classifiers ---
eeg = origeeg;
disp('training cued classifier...');

% first, rename the S16 & S17 markers according to the subsequent key-presses... (into S20|21 for S16 and S22|23 for S17)
% also, reject artifact markers (i.e. those for which the S16 marker came too early or the S17 marker came too late)
erased = [];
for m=2:length(eeg.event)
    % we have an interesting keypress
    if strcmp(eeg.event(m).type,'4') || strcmp(eeg.event(m).type,'8')
        if strcmp(eeg.event(m-1).type,'16')
            % this is supposedly a self-paced key marker, check if we are neither too late nor too early
            lat = (eeg.event(m).latency - eeg.event(m-1).latency) / eeg.srate;
            if lat >= too_early && lat <= too_late
                if strcmp(eeg.event(m).type,'4')
                    eeg.event(m-1).type = '20';
                    eeg.event(m-1).name = {'S 20'};
                else
                    eeg.event(m-1).type = '21';
                    eeg.event(m-1).name = {'S 21'};
                end
            end
            % erase these key marker in every case (since these are of no interest to us here)
            erased = [erased m];
        elseif strcmp(eeg.event(m-1).type,'17')
            % this is supposedly a cued key marker, check if we are not too late
            lat = (eeg.event(m).latency - eeg.event(m-1).latency) / eeg.srate;
            if lat < too_early % consequently early enough for a cued keypress
                if strcmp(eeg.event(m).type,'4')
                    eeg.event(m-1).type = '22';
                    eeg.event(m-1).name = {'S 22'};
                else
                    eeg.event(m-1).type = '23';
                    eeg.event(m-1).name = {'S 23'};
                end
            else
                % erase cued key markers which were too late
                erased = [erased m];
            end
        else
            % something unexpected happened in between: erase
            erased = [erased m];
        end
    end
end
% erase them
eeg.event = eeg.event(setdiff(1:length(eeg.event),erased));

% insert jittered training markers
key_markers = {'S  4', 'S  8'};
train_markers = {{'S  5', 'S  6', 'safety'}, {'S  9', 'S 10', 'safety'}}; 
nomov_markers = {'S  5', 'S  9'};
mov_markers = {'S  6', 'S 10'};
selfp_enter = 'S 16';
% training window parameters
nomov_begin = 1;    % beginning offset to the 'no-move' section, in ms
nomov_end = 1.45;   % ending of the 'no-move' section, in ms
mov_begin = -0.145; % beginning of the 'move' section, in ms relative to keypress
mov_end = -0.1;     % end of the 'move' section, in ms releative to keypress
nomov_jitter_cnt = 7; % number of jittered markers in the nomov section
mov_jitter_cnt = 7;   % number of jittered markers in the mov section

% inject jittered markers for left, right
for side = 1:2
    eeg = set_inject_jitters(eeg,{'S 20' nomov_begin nomov_end}, {nomov_jitter_cnt, {nomov_markers{side}}, 'random', 'total'});
    eeg = set_inject_jitters(eeg,{'S 21' nomov_begin nomov_end}, {nomov_jitter_cnt, {nomov_markers{side}}, 'random', 'total'});    
    eeg = set_inject_jitters(eeg,{mov_begin mov_end key_markers{side}}, {mov_jitter_cnt, {mov_markers{side}}, 'random', 'total'});
end
cue_markers = {{'S  5', 'S  6', 'safety'}, {'S  9', 'S 10', 'safety'}}; 
%cue_markers = {{'S 20' 'S 22' 'safety'}, {'S 21' 'S 23' 'safety'}};     % note: this is the version without jittered training

% optionally visualize the self-paced part of the data
if visualize
    for side = 1:2
        figure; vis_erp(bci_preproc(eeg,@para_simple_train,'window',[0 3],'events',cue_markers{side},'filter',{'noncausal-fft',[0.5 5]},'channels',channels,'laplace','full-lap'),{ 'FC3' 'FCz' 'FC4';'C3' 'Cz' 'C4';'CP3' 'CPz' 'CP4'}');
    end
    waitforbuttonpress;
end

windows = {[-0.8 -0.5; -0.7 -0.4; -0.5 0.45; -0.2 -0.175; -0.025 0], [-0.7 -0.4; -0.5 0.45; -0.3 -0.275; -0.2 -0.175; -0.025 0]};
% windows = {[0.3 0.6; 0.775 0.8; 0.875 0.9 ; 0.975 1],[0.775 0.8; 0.75 0.775]}; % note: this is the version without jittered training
classifiers = {hlp_search('lda-full',10), hlp_search('svm-poly',35)};
procX = bci_preproc(eeg,@para_simple_train,'srate',200,'laplace','no-lap','events',{'S  6' 'S 10' 'safety'},'window',[-2 0], 'filter',{'iir-lowpass',[2.5 10]},'channels',channels);
[err,classifierX,x,statsX] = bci_train(@para_simple_train,procX,{'chron' 8 8},{'chron' 8 8},[],{windows{1}},{{'lda' 0.015}}); statsX


% preprocess & train online classifiers 
if ~strcmp(filename,'geteeg')
    windows = {[-0.8 -0.5; -0.7 -0.4; -0.5 0.45; -0.2 -0.175; -0.025 0], [-0.7 -0.4; -0.5 0.45; -0.3 -0.275; -0.2 -0.175; -0.025 0]};
    % windows = {[0.3 0.6; 0.775 0.8; 0.875 0.9 ; 0.975 1],[0.775 0.8; 0.75 0.775]}; % note: this is the version without jittered training
    classifiers = {hlp_search('lda-full',10), hlp_search('svm-poly',35)};
    for side = 1:2
        proc{side} = bci_preproc(eeg,@para_simple_train,'srate',200,'laplace','no-lap','events',cue_markers{side},'window',[-2 0], 'filter',{'iir-lowpass',[2.5 10]},'channels',channels);
        [err,classifier{side},x,stats{side}] = bci_train(@para_simple_train,proc{side},{'chron' 8 8},{'chron' 8 8},[],{windows{1}},{{'lda' 0.015}}); stats{side}
    end
    % and remember them
    cla_cl = classifier{1};
    cla_cr = classifier{2};
end
eeg_cued = eeg;


% --- save the classifiers ---

if ~strcmp(filename,'geteeg')
    disp(['saving classifiers in ' filename]);
    save(filename,'cla_cl','cla_cr','cla_sl','cla_sr');
end

% optionally visualize the online performances
if visualize
    figure; vis_claperformance(eeg_cued,cla_cl,cue_markers{1}); title('left cued performance');
    figure; vis_claperformance(eeg_cued,cla_cr,cue_markers{2}); title('right cued performance');
    figure; vis_claperformance(eeg_selfp,cla_sl,train_markers{1}); title('left self-paced performance');
    figure; vis_claperformance(eeg_selfp,cla_sr,train_markers{2}); title('right self-paced performance');
end
