comparison = {'100' '200'};

% load BOTH data sets
for dataset = 1:2

    % --- load raw data ---
    channels = {'FCz','Fz','Cz','F3','C3','F4','C4','Pz'};
    samplerate = 256;

    % load it into tmp
    if dataset == 1
        if ~exist('tmp1','var')
            load C:\DEVEL\Databases\EEG\Emily\newgioData.mat newGioData
            tmp1.segments = newGioData; clear newGioData;
            load C:\DEVEL\Databases\EEG\Emily\log_1978_1_1_3.mat y
            tmp1.data = y; clear y;
        end
        tmp = tmp1;
    elseif dataset == 2
        if ~exist('tmp2','var')
            load C:\DEVEL\Databases\EEG\Emily\gio2Data.mat gio2Data
            tmp2.segments = gio2Data; clear gio2Data;
            load C:\DEVEL\Databases\EEG\Emily\log_1978_1_2_3.mat y
            tmp2.data = y; clear y;
        end
        tmp = tmp2;
    end
        

    % --- generate a data set with markers, chanlocs, etc.... ---
    num_segments = 36; % per block...

    events = struct('type',0,'latency',0);
    for block = 1:4
        for cond = 0:2
            ev = struct('type', cond*2 + 1,'latency', tmp.segments(cond*2 + 1,block)); events(end+1) = ev;
            ev = struct('type', cond*2 + 2,'latency', tmp.segments(cond*2 + 2,block)); events(end+1) = ev;
        end
    end
    for i=1:length(events)
        events(i).name = {num2str(events(i).type)};
        events(i).type = num2str(events(i).type);
    end
    events = events(2:end);

    % sort events by time
    [x,I] = sort([events.latency]); events = events(I);
    %show sorted condition order for debugging: utl_strext({events(2:2:end).type})/2-1

    % create new data set
    data = set_new('data',tmp.data(4:11,:),'srate',samplerate,'chanlocs',channels, 'event', events);
    % data.cache_term = ['subject_giovanni_session_' num2str(dataset)]; 
    
    % add window offsets
    conds = {0,1,2};
    for thecond = conds
        cond = thecond{1};
        data = set_inject_jitters(data,{num2str(cond*2+1) 5 -5 num2str(cond*2+2)}, {num_segments, num2str(cond*100), 'regular', 'total'});
    end

    % --- visualize tests ---

    % draw data!
    %data.srate = 60000;
    %pop_eegplot(data);

    % preprocess the data for analysis...

    % plot data and marker alignment
    % data = flt_envelope(flt_fast(data, 9, 12));
    % data = flt_fast(data,0,0.5);
    % figure;hold;
    % plot(data.data(8,:));
    % scatter([data.event(strcmp({data.event.type},'0')).latency],zeros(sum(strcmp({data.event.type},'0')),1), '.','g');
    % scatter([data.event(strcmp({data.event.type},'100')).latency],zeros(sum(strcmp({data.event.type},'100')),1), '.','y');
    % scatter([data.event(strcmp({data.event.type},'200')).latency],zeros(sum(strcmp({data.event.type},'200')),1), '.','r');

    % --- display bandpower.... ---
    epos = bci_preproc(data,@para_speccsp_train,'window',[-2.5 +2.5],'events',comparison,'filter','none','channels',1:8,'laplace','no-lap','srate',256);

    stddev = 1;    % turn on/off
    logdomain = 0; % turn on/off
    doplot = 0;    % turn on/off
    if doplot
        clear spec;
        classcolors = 'rgb';
        freqs = [];
        % for each channel...
        for chn = 1:length(channels)
            figure; hold;
            title(channels{chn});
            % for each class...
            for class=1:2
                subepos = find(epos.classidx==class);
                spec.(channels{chn}){class} = zeros(length(subepos), length(pwelch(epos.data(chn,:,1))));
                % for each trial in that class...
                for epo=1:length(subepos)
                    [spec.(channels{chn}){class}(epo,:),freqs] = pwelch(epos.data(chn,:,subepos(epo)),[],[],[],epos.srate);
                    if logdomain
                        spec.(channels{chn}){class}(epo,:) = log(spec.(channels{chn}){class}(epo,:));
                    end
                end
                % draw it
                m = mean(spec.(channels{chn}){class});
                s = std(spec.(channels{chn}){class});
                plot(freqs,m,classcolors(class));
                if stddev
                    plot(freqs,m+s,['-.' classcolors(class)]);
                    plot(freqs,m-s,['-.' classcolors(class)]);
                end
            end
        end
    end
    
    % --- classification analysis ---

    session{dataset} = bci_preproc(data,@para_speccsp_train,'window',[-2.5 +2.5],'events',comparison,'filter',{'fft-gausswnd',[2 40]},'channels',1:8,'laplace','no-lap','srate',256);

end

% IMPORTANT: this is a placeholder for whatever parameters you are actually using
nirsparadigm = {@para_simple_train,{'F3','C3','C4'},[-2.5 2.5]}; 
% feature-combining classifier
combinecla = {'lda' 0.015};
 % IMPORTANT: also try these...
% combinecla = 'logreg';
% combinecla = search(hlp_search('lda-full',10)); % maybe use a lower value if you don't have the time, or try multiple things on multiple machines
% combinecla = search(hlp_search('qda-full',50)); % maybe use a lower value if you don't have the time, or try multiple things on multiple machines
% combinecla = 'svmperf'; % this is ad hoc, but if it works, good...
% combinecla = search(hlp_search('svmperf-lin',50)); % this is the proper way for linear SVM
% combinecla = search(hlp_search('svmperf-rbf',3));  % this is the proper way for SVM with RBF kernel


% classify the halves...
for half=1:2
    % all training options, except for the paradigm
    opts = {'data',session{half},'outerfold',{'chron' 8},'innerfold',{'chron' 7}};
    
    % --- original EEG code, reformatted for alternative syntax ---
    [err,cla] = bci_train({'func',@para_speccsp_train,opts{:}}, 4,0,1,[],[],'lda',[2 25])
    [err,cla] = bci_train({'func',@para_speccsp_train,opts{:}}, 3,0,1,[],[],'lda',[2 25])
    [err,cla] = bci_train({'func',@para_mbcsp_train,opts{:}}, 3,[],[3 6; 9 12],'lda')
    [err,cla] = bci_train({'func',@para_mbcsp_train,opts{:}}, 4,[],[3 6; 9 12],'logreg')
    % --- combined with NIRS features ---
    [err,cla] = bci_train({'func',@para_combine_train,opts{:}}, combinecla, {{@para_speccsp_train,4,0,1,[],[],[],[2 25]},nirsparadigm})
    [err,cla] = bci_train({'func',@para_combine_train,opts{:}}, combinecla, {{@para_speccsp_train,3,0,1,[],[],[],[2 25]},nirsparadigm})
    [err,cla] = bci_train({'func',@para_combine_train,opts{:}}, combinecla, {{@para_mbcsp_train,3,[],[3 6; 9 12]},nirsparadigm})
    [err,cla] = bci_train({'func',@para_combine_train,opts{:}}, combinecla, {{@para_mbcsp_train,4,[],[3 6; 9 12]},nirsparadigm})
end


% classify session 1 vs 2:
opts = {'data',session,'outerfold',[],'innerfold',{'chron' 8}};
% --- original EEG code, reformatted for alternative syntax ---
[err,cla] = bci_train({'func',@para_speccsp_train,opts{:}}, 4,0,1,[],[],'lda',[2 25])
[err,cla] = bci_train({'func',@para_speccsp_train,opts{:}}, 3,0,1,[],[],'lda',[2 25])
[err,cla] = bci_train({'func',@para_mbcsp_train,opts{:}}, 3,[],[3 6; 9 12],'lda')
[err,cla] = bci_train({'func',@para_mbcsp_train,opts{:}}, 4,[],[3 6; 9 12],'logreg')
% --- combined with NIRS features ---
[err,cla] = bci_train({'func',@para_combine_train,opts{:}}, combinecla, {{@para_speccsp_train,4,0,1,[],[],[],[2 25]},nirsparadigm})
[err,cla] = bci_train({'func',@para_combine_train,opts{:}}, combinecla, {{@para_speccsp_train,3,0,1,[],[],[],[2 25]},nirsparadigm})
[err,cla] = bci_train({'func',@para_combine_train,opts{:}}, combinecla, {{@para_mbcsp_train,3,[],[3 6; 9 12]},nirsparadigm})
[err,cla] = bci_train({'func',@para_combine_train,opts{:}}, combinecla, {{@para_mbcsp_train,4,[],[3 6; 9 12]},nirsparadigm})


% classify across sessions (here, one needs a different classifier or better features...
% and we use 16-fold CV because we now have 16 blocks...
opts = {'data',set_merge(session{:}),'outerfold',{'chron' 16},'innerfold',{'chron' 5}};  % NOTE: you can use {'chron' 15} for the inner foldness if you have 3x the time
% --- original EEG code, reformatted for alternative syntax ---
[err,cla] = bci_train({'func',@para_speccsp_train,opts{:}}, 4,0,1,[],[],'gmm',[2 25])
[err,cla] = bci_train({'func',@para_speccsp_train,opts{:}}, 4,0,1,[],[],'lda',[2 25])
[err,cla] = bci_train({'func',@para_speccsp_train,opts{:}}, 3,0,1,[],[],'lda',[2 25])
[err,cla] = bci_train({'func',@para_mbcsp_train,opts{:}}, 3,[],[3 6; 9 12],'lda')
[err,cla] = bci_train({'func',@para_mbcsp_train,opts{:}}, 4,[],[3 6; 9 12],'logreg')
% --- combined with NIRS features ---
[err,cla] = bci_train({'func',@para_combine_train,opts{:}}, combinecla, {{@para_speccsp_train,4,0,1,[],[],[],[2 25]},nirsparadigm})
[err,cla] = bci_train({'func',@para_combine_train,opts{:}}, combinecla, {{@para_speccsp_train,3,0,1,[],[],[],[2 25]},nirsparadigm})
[err,cla] = bci_train({'func',@para_combine_train,opts{:}}, combinecla, {{@para_mbcsp_train,3,[],[3 6; 9 12]},nirsparadigm})
[err,cla] = bci_train({'func',@para_combine_train,opts{:}}, combinecla, {{@para_mbcsp_train,4,[],[3 6; 9 12]},nirsparadigm})



% get additional statistics!
%[err,cla,x,stats] = bci_train({'func',@para_speccsp_train,opts{:}}, 4,0,1,[],[],'lda',[2 25])

