function do_test_online(raweeg, processedeeg, stream_options, windowlen, percent, tolerance)
% do_test_online(Raw-EEG,Processed-EEG, Stream-Options, Window-Length, Percent, Tolerance)
% test whether the online system manages to transform the raw eeg into the processed eeg
% In:
%   Raw-EEG         : a raw eeg dataset
%   Processed-EEG   : a processed version of the raw set, as by bci_preproc (or any series of flt_* calls)
%                     or 'all', if a full test harness should be run
%   Stream-Options  : cell array of streaming options, as in ext_stream (name-value pair(s) for 'interval', 'markers', etc.)
%                     this sets up the latencies at which the comparison should be performed, default: {'interval', 1}
%   Window-Length   : length of a comparison window, relative to each tested latency, default: 1
%   Percent         : percentage of the dataset to run through, default: 100
%   Tolerance       : allowed comparison tolerance, default: 0.01
%
%                               Christian Kothe, FG MMS, TU-Berlin
%                               19.08.2009

% apply default parameters
if ~exist('processedeeg','var') processedeeg = 'all'; end
if ~exist('stream_options','var') stream_options = {'interval',1}; end
if ~exist('windowlen','var') windowlen = 1; end
if ~exist('percent','var') percent = 100; end
if ~exist('tolerance','var') tolerance = 0.1; end


% make sure that a couple of things are disabled...
if ~(isequal(0,exp_eval(@fingerprint_check)) && isequal(0,exp_eval(@fingerprint_create)) && isequal(0,exp_eval(@memoize)))
    exp_eval(exp_block({exp_set(@fingerprint_check,0),exp_set(@fingerprint_create,0),exp_set(@memoize,0)}, ...
        exp_lambda(@()do_test_online(raweeg,processedeeg,stream_options,windowlen,percent,tolerance))));
else
    
    % make sure all is evaluated
    raweeg = exp_eval(raweeg);
    processedeeg = exp_eval(processedeeg);    
    
    if ischar(processedeeg) && strcmp(processedeeg,'all')
        % --- run through various offline scenarios ---
        
        % add a special event to the raw dataset; we will use this for event-locked checking...
        tmpevent = raweeg.event([1:end end]);
        tmpevent(end).type = 'S  1';
        tmpevent(end).latency = 531;
        raweeg = set_new(raweeg, 'event', tmpevent);
        
        % run through a set of test cases....
        
        %     % --- epoched ---
        %     do_test_online(raweeg,bci_preproc(raweeg,@para_csp_train,'srate',1000, 'window',[-0.3 0.5], 'filter', {'fft-gausswnd', [7 30]},'events', {'S  1' 'S  2'}, 'channels', 1:13), 'all');
        %
        %     % stateless->stateless->stateful->stateful->epochs->epoched->epoched | set_selchans, flt_laplace, flt_lowpass, flt_bandpass, set_makepos, flt_epochs, flt_rmbase
        %     do_test_online(raweeg,flt_rmbase(flt_epochs(set_makepos(flt_bandpass(flt_lowpass(set_selchans(raweeg,{'C4' 'C3' 'Cz' 'Fz' 'O2' 'O1'}),[60 270]),[40 80 150 300]),[-0.1 0.15],{'S  1','S  2'}),[5 50], 'fft-gausswnd'),[-0.1 -0.075]) , 'all');
        %     % stateless->stateful->stateful->epochs->epoched | set_selchans, flt_lowpass, flt_bandpass, set_makepos, flt_epochs
        %     do_test_online(raweeg,flt_epochs(set_makepos(flt_bandpass(flt_lowpass(set_selchans(raweeg,{'C4' 'C3' 'Cz' 'Fz' 'O2' 'O1'}),[60 270]),[40 80 150 300]),[-0.1 0.15],{'S  1','S  2'}),[5 50], 'fft-gausswnd'), 'all');
        %     % stateless->stateful->stateful->epochs | set_selchans, flt_lowpass, flt_bandpass, set_makepos
        %     do_test_online(raweeg,set_makepos(flt_bandpass(flt_lowpass(set_selchans(raweeg,{'C4' 'C3' 'Cz' 'Fz' 'O2' 'O1'}),[60 270]),[40 80 150 300]),[-0.1 0.15],{'S  1','S  2'}), 'all');
        %     % stateless->epochs | set_selchans, set_makepos
        %     do_test_online(raweeg,set_makepos(set_selchans(raweeg,{'C4' 'C3' 'Cz' 'Fz' 'O2' 'O1'}),[-0.1 0.15],{'S  1','S  2'}), 'all');
        
        
        % --- continuous ---
        %     % stateless->stateful->stateful | set_selchans, flt_bandpass, flt_lowpass
        %     do_test_online(raweeg,flt_bandpass(flt_lowpass(set_selchans(raweeg,{'C4' 'C3' 'Cz' 'Fz' 'O2' 'O1'}),[60 270]),[40 80 150 300]), 'all');
        %     % stateless->stateful->stateless | flt_project, flt_bandpass, set_selchans
        %     do_test_online(raweeg,set_selchans(flt_bandpass(set_selchans(raweeg,{'C4' 'C3' 'Cz' 'Fz' 'O2' 'O1'}),[40 80 150 300]),{'C3' 'C4'}), 'all');
        %     % stateful->stateless->stateful | flt_bandpass, set_selchans, flt_bandpass
        %      do_test_online(raweeg,flt_bandpass(set_selchans(flt_bandpass(raweeg,[40 80 110 250]),{'C3' 'C4'}),[30 70 150 300]), 'all');
        %     % stateful->stateless | flt_bandpass, set_selchans
        %     do_test_online(raweeg,set_selchans(flt_bandpass(raweeg,[40 80 150 300]),{'C3' 'C4'}), 'all');
        %     % stateless->stateful | set_selchans, flt_bandpass
        %     do_test_online(raweeg,flt_bandpass(set_selchans(raweeg,{'C3' 'C4'}),[40 80 150 300]), 'all');
        %     % stateful | flt_bandpass
        %     do_test_online(raweeg,flt_bandpass(raweeg,[40 80 150 300]), 'all');
        %     % stateless->stateless | flt_project, set_selchans
        %     do_test_online(raweeg,set_selchans(flt_project(raweeg,magic(36)/650),{'C3' 'C4'}), 'all');
        %     % stateless | selchans
        %     do_test_online(raweeg,set_selchans(raweeg,{'C3' 'Cz' 'C4' 'Fp1'}), 'all');
        %     % stateless | flt_laplace
        %     do_test_online(raweeg,flt_laplace(raweeg), 'all'); %-> slow!
        %     % stateless | flt_project
        %     do_test_online(raweeg,flt_project(raweeg,magic(36)/650), 'all');
        
        % epoch filtering
        do_test_online(raweeg,flt_epochs(set_makepos(flt_resample(flt_resample(raweeg,[],200),[],200),[-0.5 0.6],{'S  1','S  2'}),[7 30],'fft-gausswnd',1),'all');
        % epoching
        do_test_online(raweeg,set_makepos(flt_resample(flt_resample(raweeg,[],200),[],200),[-0.5 0.6],{'S  1','S  2'}),'all');
        % two-stage resampling
        do_test_online(raweeg,flt_resample(flt_resample(raweeg,[],200),[],200),'all');
        % one-stage resampling
        do_test_online(raweeg,flt_resample(raweeg,[],200),'all');
        % raw
        do_test_online(raweeg,raweeg, 'all');
        
        disp('test harness ran with no errors.');
        return;
    end
    
    if ischar(stream_options) && strcmp(stream_options,'all')
        % --- run through various online scenarios ---
        if size(processedeeg.data,3) == 1
            % continuous-data tests
            do_test_online(raweeg,processedeeg,{'interval',200},400);   % detector window > rebuffer interval
            do_test_online(raweeg,processedeeg,{'interval',200},100);   % detector window < rebuffer interval
            do_test_online(raweeg,processedeeg,{'interval',200},200);   % detector window = rebuffer interval
            do_test_online(raweeg,processedeeg,{'interval',200},1);     % detector window is 1
            do_test_online(raweeg,processedeeg,{'interval',2},200,2.5); % rebuffer interval is 2
            do_test_online(raweeg,processedeeg,{'interval',1},200,5); % rebuffer interval is 1
        else
            % epoch-data tests
            do_test_online(raweeg,processedeeg,{'interval',1},size(processedeeg.data,2)); % rebuffer interval is 1
        end
        return;
    end
    
    % --- construct a comparison "detector" which we use to compare the offline processed eeg set with an online processes version thereof ---
    if size(processedeeg.data,3) == 1
        % we are operating on continuous data
        det.bcilab.prediction_function = @comparer_func_cont;          % the function which we use to do the comparison
    else
        % we have to do epoch-based comparison
        det.bcilab.prediction_function = @comparer_func_epo;          % the function which we use to do the comparison
        % find the event offset relative to the window...
        det.eventoffset = find(processedeeg.times==0)-1;
    end
    
    
    det.original = raweeg;                                                  % the original dataset
    det.reference = processedeeg;                                           % the reference set against which we compare
    det.bcilab.window_length = windowlen;                                   % the length of the comparison windows
    det.bcilab.online_expression = processedeeg.bcilab.online_expression;   % the processing pipeline to obtain the offline processed set
    det.tolerance = tolerance;                                              % the allowed comparison tolerance
    
    % --- stream the set through the comparison detector ---
    onl_stream(raweeg, det, 'show_progress', 1, 'percent',percent, stream_options{:});
    
end





% a comparator function for online processed data versus offline processed data, epoch-based
% this actually only works when the rebuffer interval is 0
function res = comparer_func_epo(set, det)

% our sample-correct window into the reference dataset...
wnd = (set.smax-size(set.data,2)+1) : set.smax;

% if our window had been generated by epoching, this would have been the time-locking event's latency:
evlat = wnd(1)+det.eventoffset;
% here we test of one of the events in the epoched reference dataset match our "imaginary" event... (to compare the epoch data)
evhit = evlat == [det.reference.epoch.latency];
% found one...
if any(evhit) && size(wnd,2) == size(det.reference.data,2)
    try
        % get its index
        evnum = find(evhit);
        % compare the data
        if ~all(all(abs(set.data - det.reference.data(:,:,evnum))) < det.tolerance)
            error('comparison failed');
        else
            disp(['matching epoch at ' num2str(evlat)]);
        end
    catch
        warning('sdf:dsdsf','indexing out of range or number of channels invalid...');
    end
end
res = 0;


% a comparator function for online processed data versus offline processed data, continuous
function res = comparer_func_cont(set, det)
wnd = (set.smax-size(set.data,2)+1) : set.smax;
try
    if ~all(all(abs(set.data - det.reference.data(:,wnd)) < det.tolerance))
        error('comparison failed');
    end
catch
    error('indexing out of range or number of channels invalid...');
end
res = 0;

