function eeg = flt_attach_LLt(eeg, modelpath, smoothlen)
% attaches the per-model log-likelihood as new channels in the dataset
% should be run directly after an appropriate highpass

if ~exp_begindef('filter') return; end
if ~exist('smoothlen','var') smoothlen = 7.5; end

% subset, center and sphere the data using original model parameters
origm = loadmodout(modelpath);
load([modelpath '/chanlocs.mat']);
data = double(eeg.data(chaninds,:));
data = data - repmat(mean(data,2),1,size(data,2));
data = origm.S*data;

% then deactivate sphering etc in the mode
origm.data_mean = zeros(size(origm.data_mean));
origm.S = eye(size(origm.S));
% and compute smoothed likelihoods
smoothwnd = hann(smoothlen*eeg.srate); smoothwnd = smoothwnd/sum(smoothwnd);
LLt = filtfilt_fast(smoothwnd,1,getmodLLt(data,origm)')';

 
% % write the data -- alternative way to compute the above results...
% floatwrite(data,[modelpath '/tempdata.fdt']);
% 
% % write control parameters
% copyfile([modelpath '/input.reference'], [modelpath '/input.param']);
% % append field_dim line
% try
%     f = fopen([modelpath '/input.param'],'a');
%     fprintf(f,'field_dim %d\n',size(eeg.data,2));
% catch,end
% fclose(f);
% 
% % clear contents of the out directory
% delete([modelpath '/out/*']);
% 
% % run amica to get LLt's
% system(['/data/common/amica/amica ' fullfile(modelpath,'input.param')]);
% 
% % load LLt's and filter them 
% m = loadmodout([modelpath '/out']);
% smoothwnd = hann(smoothlen*eeg.srate); smoothwnd = smoothwnd/sum(smoothwnd);
% LLt = filtfilt_fast(smoothwnd,1,m.LLt')';

% map to probabilities
v = zeros(size(LLt));
for k = 1:size(v,1)
    v(k,:) = 1./sum(exp(bsxfun(@minus,LLt,LLt(k,:))),1); end
figure;plot(v')

% attach to dataset
eeg.data = [eeg.data; v];
eeg.nbchan = size(eeg.data,1);
channames = {};
for k=1:origm.num_models
    channames{end+1} = ['prob' num2str(k)]; end
[eeg.chanlocs(end+1 : end+origm.num_models).labels] = channames{:};

exp_enddef;


% usage:
%a = exp_eval(flt_fir(io_loadset('data:/concerto/tim-session3/tim-session3.bdf'),[],[0.5 2],'hp','zero-phase'))
%b = exp_eval(flt_attach_LLt(a,'/data/projects/christian/Concerto/6models'));
