function eeg = flt_specdecomp(eeg, specdecomp, compidx)
% Output-Set = flt_fourier(Input-Set, IC-Spectral-Decomposition, Component-Subset)
% Transform an epoched dataset into an ic Spectral decomposition
%
% In:
%   Input-Set      :   Epoched eeg set to be processed
%
%
% Out: 
%   Output-Set  :   baseline-corrected eeg set
%
%                   Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                   2010-03-28

if ~exp_begindef('filter') return; end

icsd = specdecomp;

% spectral decomposition given as a filename
if ischar(icsd)    
    icsd = {icsd,'icSpectralDecomposition'}; end
% spectral decomposition given as cell array of file name and variable name
if iscell(icsd)
    icsd = getfield(load(icsd{1}),icsd{2}); end

if exist('compidx','var') && ~isempty(compidx)
    icsd = icsd(compidx); end

% assume that the data is pre-projected
eeg.icaact = eeg.data;

% calculate ic spectral decompositons and put them back in icsd field.
newdata = zeros(length(icsd),icsd(1).numberOfSpctralTemplates,size(eeg.data,3));
for i=1:length(icsd)
    icsd(i) = ic_spectral_decomposition_activity_on_new_data(eeg, icsd(i));
    tmp = accumulate_ic_spectral_decomposition_activations(icsd(i));    
    try
        tmp = squeeze(mean(tmp,2));
        newdata(i,1:size(tmp,1),:) = tmp; % sometimes variables are pruned by the IM code
    catch
        1
    end
end

eeg.data = newdata;
eeg.pnts = size(eeg.data,2);

exp_enddef;