% Christian,
%    Load the data matrix PCA_data0606.mat, you will find  
%  
%   Group_PowerMatrix_index                177x1              1416  double             
%   PowerMatrix_learning_input_PCA_P1      177x13            18408  double             
%   PowerMatrix_learning_input_PCA_P2      177x13            18408  double             
%   PowerMatrix_learning_input_PCA_P3      177x13            18408  double             
%   PowerMatrix_learning_input_PCA_P4      177x13            18408  double             
%  
% Total number of trials : 177
% index '0' in Group_PowerMatrix_index means the feedback was effective. index '1' means feedback was not effective.
% Data files contain the projections along 4 principal components.
%  
% TP
%
% Class 1: ~80% of the data, not effective 
% Class 0: ~20% of the data, effective (predicted by baseline)


% load data
cd /data/projects/christian/PCA_data0606
load PCA_data0606


% organize 
trials  = [PowerMatrix_learning_input_PCA_P1 PowerMatrix_learning_input_PCA_P2 PowerMatrix_learning_input_PCA_P3 PowerMatrix_learning_input_PCA_P4];
targets = Group_PowerMatrix_index;


% subset to classes 0/1 - what about class 2?
trials = trials(targets~=2,:);
targets = targets(targets~=2);


% various methods to be tested
methods = {{'lda'},...                                          % shrinkage LDA
           {'qda'},...                                          % shrinkage QDA (actually non-linear...)
           {'logreg' [] 'variant' 'vb-iter'},...                % variational Bayes logistic regression
           {'logreg' [] 'variant' 'vb-ard'}, ...                % variational Bayes sparse logistic regression
           {'svmlinear' search(2.^(-5:2:15))},...               % linear SVM
           {'logreg' search(2.^(-6:1:10)) 'variant' 'l1'}, ...  % sparse logistic regression
           ...% -- if all of the above fails, the data is not linearly separable ---
           ...
           {'svmperf' search(2.^(-5:2:15)) 'kernel' 'rbf' 'g' search(-15:2:6)}, ...     % kernel support vector machines
           {'hkl' [] 'kernel' 'hermite'},...                                            % hierarchical kernel learning
           {'hkl' [] 'kernel' 'anova'},...                                              % dito
           {'hkl' [] 'kernel' 'spline'},...                                             % dito
           {'rvm' 'gamma' search(2.^(-16:2:4))},...                                     % Relevance vector machine
           {'gmm' 1 'variant' 'gem'}};                                                  % gaussian mixture models
           % ----- if all of the above fails, the data is not non-linearly separable without major additional assumptions ---

           
% evaluate
for m=1:length(methods)
    fprintf('testing method %d...\n',m);
    try
        [loss,stats] = utl_nested_crossval({trials,targets}, 'args',methods{m})
        
        losses{m} = loss; statistics{m} = stats; pause(1); 
    catch e
        env_handleerror(e);
    end
end
