function r = process_concerto_bci_multiclass_advance(varargin)
global bcilab;
global results; results = [];

opts = hlp_varargin2struct(varargin,'paradigm','widecsp','comparison','sess1-firsthalf-vs-secondhalf','online',1);

allchans = readlocs('yellow-cap.elp');
chansel = setdiff({allchans(1:128).labels},{'B19','B29','D14','A18','D32','A32'});

load rejection-weights.mat
switch opts.paradigm
    case 'modulators'
        load(env_translatepath('data:/concerto/tim-session1/independentModulator_brain_small.mat'));
        paradigm = @para_modulators; options = {'learner','rvm','channels',cleanchans,'window',[-1 1],'modulators',independentModulator,opts}; % independent modulators
    case 'modulators-svm'
        load(env_translatepath('data:/concerto/tim-session1/independentModulator_brain_small.mat'));
        paradigm = @para_modulators; options = {'learner','svmlinear','channels',cleanchans,'window',[-1 1],'modulators',independentModulator,opts}; % independent modulators
    case 'modulators-slogreg'
        load(env_translatepath('data:/concerto/tim-session1/independentModulator_brain_small.mat'));
        paradigm = @para_modulators; options = {'learner',{'logreg',1,'variant','l1'},'channels',cleanchans,'window',[-1 1],'modulators',independentModulator,opts}; % independent modulators
    case 'cleancsp'
        paradigm = @para_csp; options = {'srate',[],'channels',cleanchans,'project',wts(bcomps,:)*sph,'window',[-1 1],'filter',{'fft-nownd',[8 15]},opts}; % hardcore artifact-rejected CSP
    case 'cleanspeccsp'
        paradigm = @para_speccsp; options = {'srate',[],'channels',cleanchans,'project',wts(bcomps,:)*sph,'window',[-1 1],'filter',{'fft-nownd',[8 15]},opts}; % hardcore artifact-rejected CSP
    case 'widecsp'
        paradigm = @para_csp; options = {'srate',[],'window',[-1 1], 'filter',{'fft-nownd',[8 15 20 35 100 200]},opts}; 
    case 'mbcsp'
        paradigm = @para_multiband_csp; options = {'srate',[],'window',[-1 1], 'bands',{{'fft-nownd',[8 15]},{'fft-nownd',[20 35]},{'fft-nownd',[100 200]}},opts}; 
    case 'ultrawidecsp'
        paradigm = @para_csp; options = {'srate',[],'channels',chansel,'window',[-1 1], 'filter',{'fft-nownd',[8 55 65 200]},opts}; 
    case 'widecleancsp'
        paradigm = @para_csp; options = {'srate',[],'channels',cleanchans,'project',wts(bcomps,:)*sph,'window',[-1 1], 'filter',{'fft-nownd',[8 15 20 35 100 200]},opts}; 
    case 'ultrawidecleancsp'
        paradigm = @para_csp; options = {'srate',[],'channels',cleanchans,'project',wts(bcomps,:)*sph,'window',[-1 1], 'filter',{'fft-nownd',[8 55 65 200]},opts}; 
    case 'mbcleancsp'
        paradigm = @para_multiband_csp; options = {'srate',[],'channels',cleanchans,'project',wts(bcomps,:)*sph,'window',[-1 1],'bands',{{'fft-nownd',[8 15]},{'fft-nownd',[20 35]},{'fft-nownd',[100 200]}},opts}; 
    case 'vanillacsp'
        paradigm = @para_csp; options = {'srate',[],'window',[-1 1], 'filter',{'fft-nownd',[8 15]},opts}; 
    case 'icbp'
        paradigm = @para_bandpower; options = {'laplace','no-lap','srate',[],'channels',cleanchans,'project',wts(bcomps,:)*sph,'window',[-1 1],'filter',{'fft-nownd',[8 15 20 35 100 200]},opts}; % hardcore artifact-rejected CSP
    otherwise
        error('unknown paradigm');
end

if iscell(opts.comparison)
    events = {{{'A_h1','A_h2'},{'B_h1','B_h2'},{'C_h1','C_h2'},{'F_h1','F_h2'},{'H_h1','H_h2'}}, {{'A_h1','A_h2'},{'B_h1','B_h2'},{'C_h1','C_h2'},{'F_h1','F_h2'},{'H_h1','H_h2'}}};
    eeg = cellfun(@concerto_load_raw_reduced,opts.comparison,'UniformOutput',false);
    if length(eeg) > 2
        for i=1:length(eeg)
            eeg{i} = flt_selchans(eeg{i},cleanchans); end
        %eeg = cellfun(@(data)flt_selchans(data,cleanchans),eeg,'UniformOutput',false);
        eeg = {set_merge(eeg{1:end-1}),eeg{end}}; 
    end
else
    error('unknown comparison');
end

% turn on memory & disk cache, turn off unnecessary checks
exp_eval(Set(@memoize,{'memory',1,'disk',1}));
exp_eval(Set(@fingerprint_check,0));
exp_eval(Set(@fingerprint_create,0));

if ispc
    % on Windows, we cut the mem cache capacity to 1GB, to be sure.
    bcilab.cache.capacity = 1000000000; 
else
    [ret,name] = system('hostname');
    % on the laptop, we use 4GB (so we can run multiple matlabs)
    if strcmp(name,'Dellos') 
        bcilab.cache.capacity = 4000000000; end
    % on compute nodes, we use 6gb
    if ~isempty(strfind(name,'compute-'))
        bcilab.cache.capacity = 6000000000; end
end

% train
[r.trainerr,r.traincla,r.trainstats] = bci_train({'func',paradigm,'data',eeg{1},'eval_scheme',0},'events',events{1},options{:});

if length(eeg) > 1
    % more than one set: predict & display results
    [r.preds,r.testerr,r.teststats] = bci_predict(r.traincla,bci_preproc(eeg{end},paradigm,'events',events{end},options{:}), 'process',0);
    figure;imagesc(filtfilt(ones(1,100),1,r.preds{2})); title('offline');
    % test the online system & display results
    if opts.online
        [r.predsom,r.latsom] = onl_stream(eeg{end},r.traincla,'markers',[events{end}{:}],'show_progress',1,'offset',512);
        figure;imagesc(filtfilt(ones(1,100),1,r.predsom{2}(1:end,:))); title('online');
        preds = r.predsom;
    end
end

% save properly
classifier = r.traincla;
save emo-classifier3.mat classifier

results = r;
