function r = process_concerto_bci_multiclass_reduce(varargin)
global bcilab;
global results; results = [];

opts = hlp_varargin2struct(varargin,'paradigm','cleancsp','comparison','sess1-firsthalf-vs-secondhalf','online',1);

switch opts.paradigm
    case 'modulators'
        load /data/projects/concerto/rejection-weights.mat
        load /data/projects/concerto/tim-session1/independentModulator_brain_small.mat        
        %paradigm = @para_modulators; options = {'learner','lda','channels',cleanchans,'window',[-1 1],'modulators',independentModulator,opts}; % independent modulators
        paradigm = @para_modulators; options = {'channels',cleanchans,'window',[-1 1],'modulators',independentModulator,opts}; % independent modulators        
    case 'cleancsp'
        load /data/projects/concerto/rejection-weights.mat
        paradigm = @para_csp; options = {'srate',[],'channels',cleanchans,'project',wts(bcomps,:)*sph,'window',[-1 1],'filter',{'fft-nownd',[8 15]},opts}; % hardcore artifact-rejected CSP
    case 'cleanspeccsp'
        paradigm = @para_speccsp; options = {'srate',[],'channels',cleanchans,'project',wts(bcomps,:)*sph,'window',[-1 1],'filter',{'fft-nownd',[8 15]},opts}; % hardcore artifact-rejected CSP
    case 'vanillacsp'
        paradigm = @para_csp; options = {'srate',[],'window',[-1 1], 'filter',{'fft-nownd',[8 15]},opts}; % vanilla CSP
    otherwise
        error('unknown paradigm');
end

iv = {'A','B','C','F','H'};  % names of the individual intervals
if iscell(opts.comparison)
    events = {{{'A_h1','A_h2'},{'B_h1','B_h2'},{'C_h1','C_h2'},{'F_h1','F_h2'},{'H_h1','H_h2'}}, {{'A_h1','A_h2'},{'B_h1','B_h2'},{'C_h1','C_h2'},{'F_h1','F_h2'},{'H_h1','H_h2'}}};
    eeg = cellfun(@concerto_load_raw_reduced,opts.comparison,'UniformOutput',false);
    if length(eeg) > 2
        eeg = {set_merge(eeg{1:end-1}),eeg{end}}; end
elseif ischar(opts.comparison)
    switch opts.comparison
%         case 'sess1-vs-sess1'
%             events = {{{'A_h1','A_h2'},{'B_h1','B_h2'},{'C_h1','C_h2'},{'F_h1','F_h2'},{'H_h1','H_h2'}}, {{'A_h1','A_h2'},{'B_h1','B_h2'},{'C_h1','C_h2'},{'F_h1','F_h2'},{'H_h1','H_h2'}}};
%             eeg = {concerto_load_raw_reduced(1), concerto_load_raw_reduced(1)};
        otherwise
            error('unknown comparison');
    end
else
    error('unknown comparison');
end

% turn on memory & disk cache, turn off unnecessary checks
exp_eval(Set(@memoize,{'memory',1,'disk',1}));
exp_eval(Set(@fingerprint_check,0));
exp_eval(Set(@fingerprint_create,0));

if ispc
    % on Windows, we cut the mem cache capacity to 1GB, to be sure.
    bcilab.cache.capacity = 1000000000; end    

% train
[r.trainerr,r.traincla,r.x,r.trainstats] = bci_train({'func',paradigm,'data',eeg{1},'eval_scheme',0},'events',events{1},options{:});

if length(eeg) > 1
    % more than one set: predict & display results
    [r.preds,r.testerr,r.teststats] = bci_predict(r.traincla,bci_preproc(eeg{end},paradigm,'events',events{end},options{:}), 'process',0);
    figure;imagesc(filtfilt(ones(1,100),1,r.preds{2})); title('offline');
    % test the online system & display results
    if opts.online
        [r.predsom,r.latsom] = onl_stream(eeg{end},r.traincla,'markers',[events{end}{:}],'show_progress',1,'offset',512);
        figure;imagesc(filtfilt(ones(1,100),1,r.predsom{2}(1:end,:))); title('online');
    end
end

% save properly
save emo-classifier3.mat r 

results = r;




% dare to include the other two conditions?
% events_half1 = {'A_h1','B_h1','C_h1','D_h1','E_h1','F_h1','G_h1','H_h1'}; 
% events_half2 = {'A_h1','B_h2','C_h2','D_h2','E_h2','F_h2','G_h2','H_h2'};

% % train on the first half
% [r.trainerr,r.traincla,r.x,r.trainstats] = bci_train({'func',paradigm,'data',eeg{1}},'events',events_half1,options{:});
% % predict on the second half
% [r.preds,r.testerr,r.teststats] = bci_predict(r.traincla,bci_preproc(eeg{1},paradigm,'events',events_half2,options{:}), 'process',0);
% % display results
% figure;imagesc(filtfilt(ones(1,100),1,r.preds{2})); title('other half');
% 
% % train on the whole session
% [r.trainerr,r.traincla,r.x,r.trainstats] = bci_train({'func',paradigm,'data',eeg{1}},'events',events_all,options{:});
% % predict on the same and the other session
% [r.predssame,r.testerrsame,r.teststatssame] = bci_predict(r.traincla,eeg{1});
% [r.predsother,r.testerrother,r.teststatsother] = bci_predict(r.traincla,eeg{2});
% % display results
% figure;imagesc(filtfilt(ones(1,100),1,r.predssame{2})); title('same session');
% figure;imagesc(filtfilt(ones(1,100),1,r.predsother{2})); title('other session');
