function process_concerto_bci_onlinetest(varargin)
global bcilab;
global results; results = [];

opts = hlp_varargin2struct(varargin,'paradigm','cleancsp','comparison','sess1-firsthalf-vs-secondhalf');

switch opts.paradigm
    case 'modulators'
        load /data/projects/concerto/rejection-weights.mat
        load /data/projects/concerto/tim-session1/independentModulator_brain_small.mat        
        %paradigm = @para_modulators; options = {'learner','lda','channels',cleanchans,'window',[-1 1],'modulators',independentModulator,opts}; % independent modulators
        paradigm = @para_modulators; options = {'channels',cleanchans,'window',[-1 1],'modulators',independentModulator,opts}; % independent modulators        
    case 'cleancsp'
        load /data/projects/concerto/rejection-weights.mat
        paradigm = @para_csp; options = {'channels',cleanchans,'project',wts(bcomps,:)*sph,'window',[-1 1],'filter',{'fft-nownd',[8 15]},opts}; % hardcore artifact-rejected CSP
    case 'cleanspeccsp'
        paradigm = @para_speccsp; options = {'channels',cleanchans,'project',wts(bcomps,:)*sph,'window',[-1 1],'filter',{'fft-nownd',[8 15]},opts}; % hardcore artifact-rejected CSP
    case 'vanillacsp'
        paradigm = @para_csp; options = {'window',[-1 1], 'filter',{'fft-nownd',[8 15]},opts}; % vanilla CSP
    otherwise
        error('unknown paradigm');
end

switch opts.comparison
    case 'sess1-firsthalf-vs-secondhalf'
        events = {{'B_h1','C_h1','E_h1','F_h1','G_h1','H_h1'}, {'B_h2','C_h2','E_h2','F_h2','G_h2','H_h2'}};
        eeg = {concerto_load_raw(1), concerto_load_raw(1)};
    case 'sess1-secondhalf-vs-firsthalf'
        events = {{'B_h2','C_h2','E_h2','F_h2','G_h2','H_h2'},{'B_h1','C_h1','E_h1','F_h1','G_h1','H_h1'}};
        eeg = {concerto_load_raw(1), concerto_load_raw(1)};
    case 'sess1-vs-sess2'
        events = {{{'B_h1','B_h2'},{'C_h1','C_h2'},{'E_h1','E_h2'},{'F_h1','F_h2'},{'G_h1','G_h2'},{'H_h1','H_h2'}}, {{'B_h1','B_h2'},{'C_h1','C_h2'},{'E_h1','E_h2'},{'F_h1','F_h2'},{'G_h1','G_h2'},{'H_h1','H_h2'}}};
        eeg = {concerto_load_raw(1), concerto_load_raw(2)};
    case 'sess2-vs-sess1'
        events = {{{'B_h1','B_h2'},{'C_h1','C_h2'},{'E_h1','E_h2'},{'F_h1','F_h2'},{'G_h1','G_h2'},{'H_h1','H_h2'}}, {{'B_h1','B_h2'},{'C_h1','C_h2'},{'E_h1','E_h2'},{'F_h1','F_h2'},{'G_h1','G_h2'},{'H_h1','H_h2'}}};
        eeg = {concerto_load_raw(2), concerto_load_raw(1)};
    otherwise
        error('unknown comparison');
end

% turn on memory & disk cache, turn off unnecessary checks
exp_eval(Set(@memoize,{'memory',1,'disk',1}));
exp_eval(Set(@fingerprint_check,0));
exp_eval(Set(@fingerprint_create,0));

if ispc
    % on Windows, we cut the mem cache capacity to 1GB, to be sure.
    bcilab.cache.capacity = 1000000000; end    

% train
[r.trainerr,r.traincla,r.x,r.trainstats] = bci_train({'func',paradigm,'data',eeg{1},'eval_scheme',0},'events',events{1},options{:});

% predict
[r.preds,r.testerr,r.teststats] = bci_predict(r.traincla,bci_preproc(eeg{2},paradigm,'events',events{2},options{:}), 'process',0);

% display results
figure;imagesc(filtfilt(ones(1,100),1,r.preds{2})); title('other half');

% test the online system...
[r.predso,r.latso] =  onl_stream(eeg{1},r.traincla,'interval',50,'show_progress',1);

results = r;




% dare to include the other two conditions?
% events_half1 = {'A_h1','B_h1','C_h1','D_h1','E_h1','F_h1','G_h1','H_h1'}; 
% events_half2 = {'A_h1','B_h2','C_h2','D_h2','E_h2','F_h2','G_h2','H_h2'};

% % train on the first half
% [r.trainerr,r.traincla,r.x,r.trainstats] = bci_train({'func',paradigm,'data',eeg{1}},'events',events_half1,options{:});
% % predict on the second half
% [r.preds,r.testerr,r.teststats] = bci_predict(r.traincla,bci_preproc(eeg{1},paradigm,'events',events_half2,options{:}), 'process',0);
% % display results
% figure;imagesc(filtfilt(ones(1,100),1,r.preds{2})); title('other half');
% 
% % train on the whole session
% [r.trainerr,r.traincla,r.x,r.trainstats] = bci_train({'func',paradigm,'data',eeg{1}},'events',events_all,options{:});
% % predict on the same and the other session
% [r.predssame,r.testerrsame,r.teststatssame] = bci_predict(r.traincla,eeg{1});
% [r.predsother,r.testerrother,r.teststatsother] = bci_predict(r.traincla,eeg{2});
% % display results
% figure;imagesc(filtfilt(ones(1,100),1,r.predssame{2})); title('same session');
% figure;imagesc(filtfilt(ones(1,100),1,r.predsother{2})); title('other session');
