function [res,trainloss,testloss,predictions] = process_concerto_bci(varargin)
opts = hlp_varargin2struct(varargin,'paradigm','ultrawidecsp','comparison',{3,4},'online',1);
global r results; results = [];
global bcilab;
x=min(1); %#ok<*NASGU>


myevents = {{'A_h1','A_h2'},{'B_h1','B_h2'},{'C_h1','C_h2'},{'F_h1','F_h2'},{'H_h1','H_h2'}};

eeg = io_loadset('data:/concerto/tim-session3/raw.set');
[trainloss,model] = bci_train({'func',@para_csp,'data',eeg},'events',myevents,'channels',1:128,'epoch',[-1 1],'spectrum',[8 55; 65 200]);

eeg2 = io_loadset('data:/concerto/tim-session4/raw.set');
[predictions,testloss,stats] = bci_predict(model,eeg2);








%  [loss,model] = bci_train({'func',@para_speccsp,'data',eeg}, 'events',myevents, 'channels',1:128, 'epoch',[-1 1], ...
%      'spectrum',[8 60], 'prior',@(f) f>8&f<55, 'pp',search(-1:0.5:1),'pq',search(0:0.5:4), ...
%      'learner', {'logreg',search(2.^(-6:0.5:10)),'variant','l1'});



% select paradigm options
allchans = hlp_microcache('test',@readlocs,'yellow-cap.elp'); chansel = setdiff({allchans(1:128).labels},{'B19','B29','D14','A18','D32','A32'});
switch opts.paradigm
    case 'ultrawidecsp'
        paradigm = @para_csp; options = {'srate',[], 'channels',1:128, 'epoch',[-1 1], 'spectrum',[8 55; 65 200], opts}; 
    case 'ultrawidespeccsp'
        flt = @(f) f>8 & f<150 & ~(f>55 & f<65);
        paradigm = @para_speccsp; options = {'srate',300, 'channels',1:128, 'epoch',[-1 1], 'spectrum',[8 150], 'prior',flt, opts}; 
    case 'speccsp'
        flt = @(f) f>8 & f<55;
        paradigm = @para_speccsp; options = {'srate',150, 'channels',1:128, 'epoch',[-1 1], 'spectrum',[8 60], 'prior',flt, opts}; 
    case 'speccsplr'
        flt = @(f) f>8 & f<55;
        paradigm = @para_speccsp; options = {'srate',150, 'channels',1:128, 'epoch',[-1 1], 'spectrum',[8 60], 'prior',flt, 'learner',{'logreg',1,'variant','l2'},opts}; 
    case 'ultrawidespeccsplr'
        flt = @(f) f>8 & f<150 & ~(f>55 & f<65);
        paradigm = @para_speccsp; options = {'srate',300, 'channels',1:128, 'epoch',[-1 1], 'spectrum',[8 150], 'prior',flt, 'learner',{'logreg',1,'variant','l2'}, opts}; 
    case 'csplr'
        paradigm = @para_csp; options = {'srate',[], 'channels',1:128, 'epoch',[-1 1], 'spectrum',[7 55], 'learner',{'logreg',1,'variant','l2'}, opts}; 
    case 'dal'
        paradigm = @para_dal_hf; options = {'srate',150, 'channels',1:128, 'epoch',[-1 1], 'spectrum',[7 55],opts}; 
    otherwise
        error('unknown paradigm');
end

% select data and markers
events = {{{'A_h1','A_h2'},{'B_h1','B_h2'},{'C_h1','C_h2'},{'F_h1','F_h2'},{'H_h1','H_h2'}}, {{'A_h1','A_h2'},{'B_h1','B_h2'},{'C_h1','C_h2'},{'F_h1','F_h2'},{'H_h1','H_h2'}}};
eeg = cellfun(@concerto_load_raw_reduced,opts.comparison,'UniformOutput',false);
if length(eeg) > 2
    for i=1:length(eeg)
        eeg{i} = flt_selchans(eeg{i},cleanchans); end
    eeg = {set_merge(eeg{1:end-1}),eeg{end}};
end

% turn on memory & disk cache, turn off unnecessary checks
exp_eval(Set(@memoize,{'memory',0,'disk',1}));
exp_eval(Set(@fingerprint_check,0));
exp_eval(Set(@fingerprint_create,0));

% set cache capacity, depending on machine
if ispc
    bcilab.cache.capacity = 1000000000; 
else
    [ret,name] = system('hostname'); %#ok<ASGLU>
    if strcmp(name,'Dellos') 
        bcilab.cache.capacity = 4000000000; end
    if ~isempty(strfind(name,'compute-'))
        bcilab.cache.capacity = 1000000000; end % was 6gb
end

% remove bad channels...
eeg{1} = flt_clean_channels(eeg{1});

% train
[r.trainerr,r.traincla,r.trainstats] = bci_train({'func',paradigm,'data',eeg{1},'eval_scheme',0},'events',events{1},options{:});

% test 
[r.preds,r.testerr,r.teststats] = bci_predict(r.traincla,eeg{end});

% plot time course
figure('position',[0 0 1600 300]);imagesc(filtfilt(ones(1,10),1,r.preds{2})');
title('Trained on session 3, tested on session 4');
ylabel('Predicted Distribution');
xlabel('Session time')
save2pdf('concerto-spec-3vs4.pdf',gcf)

% plot Spec-CSP scalp maps
figure('position',[0 0 1600 800])
comps = {[1 2] [1 3] [1 4] [1 5] [2 3] [2 4] [2 5] [3 4] [3 5] [4 5]};
for c = 1:length(comps)
    for p = 1:6
        subplot(6,length(comps),(p-1)*length(comps)+c);
        topoplot(r.traincla.voting{comps{c}(1),comps{c}(2)}.featuremodel.W(:,p),r.traincla.bcilab.channel_locations); 
        title(num2str(p));
        camzoom(1.75);
    end
end

% plot spectral filters
filtlen = 8;
figure('position',[0 0 1600 800])
comps = {[1 2] [1 3] [1 4] [1 5] [2 3] [2 4] [2 5] [3 4] [3 5] [4 5]};
for c = 1:length(comps)
    for p = 1:6
        subplot(6,length(comps),(p-1)*length(comps)+c);
        m = r.traincla.voting{comps{c}(1),comps{c}(2)}.featuremodel;
        plot(m.freqs(m.bands),filtfilt(ones(1,filtlen),1,m.alpha(m.bands,p)));
        %topoplot(r.traincla.voting{comps{c}(1),comps{c}(2)}.featuremodel.W(:,p),r.traincla.bcilab.channel_locations); 
        title(num2str(p));
    end
end

%     % test the online system & display results
%     if opts.online
%         [r.predsom,r.latsom] = onl_stream(eeg{end},r.traincla,'markers',[events{end}{:}],'show_progress',1,'offset',512);
%         figure;imagesc(filtfilt(ones(1,100),1,r.predsom{2}(1:end,:))); title('online');
%         preds = r.predsom;
%     end


% save properly
classifier = r.traincla;
save emo-classifier-new.mat classifier

results = r;
res = r;


% plot CSP scalp maps
figure('position',[0 0 1600 800])
comps = {[1 2] [1 3] [1 4] [1 5] [2 3] [2 4] [2 5] [3 4] [3 5] [4 5]};
for c = 1:length(comps)
    for p = 1:6
        subplot(6,length(comps),(p-1)*length(comps)+c);
        topoplot(r.traincla.voting{comps{c}(1),comps{c}(2)}.featuremodel(:,p),r.traincla.bcilab.channel_locations); 
        title(num2str(p));
        camzoom(1.75);
    end
end



% r=process_concerto_bci('comparison',{3,4},'paradigm','speccsplr') -23% error