function process_concerto_bci
global bcilab;
global results; results = [];

% the paradigm to use and its custom options
paradigm = @para_csp; options = {'window',[-1 1], 'filter',{'fft-nownd',[8 15]}};

% the intervals of interest, in our subsequent analysis (the empty ones are trash and omitted)
iv = {'','B','C','','E','F','G','H'};

% computes event-types for the given intervals i,j (optionally for just a half of the session)
full_session = @(i,j) {{[iv{i} '_h1'],[iv{i} '_h2']}, {[iv{j} '_h1'],[iv{j} '_h2']}};
half_session = @(i,j,half) {[iv{i} '_h' half], [iv{j} '_h' half]};


% load session 1 & 2 with markers
eeg = {concerto_load_raw(1), concerto_load_raw(2)};

% turn on memory & disk cache, turn off unnecessary checks
exp_eval(Set(@memoize,{'memory',1,'disk',1}));
exp_eval(Set(@fingerprint_check,0));
exp_eval(Set(@fingerprint_create,0));

if ispc
    % on Windows, we cut the mem cache capacity to 1GB, to be sure.
    bcilab.cache.capacity = 1000000000; end
    
for i = 1:length(iv)
    for j = i+1:length(iv)
        if isempty(iv{i}) || isempty(iv{j})
            continue; end
        disp([ num2str(i) ' -vs- ' num2str(j)]);
        
        % train on first full session
        [r.s1_err,r.s1_cla,x,r.s1_stat] = bci_train({'func',paradigm,'data',eeg{1}},'events',full_session(i,j),options{:});
        % test on the second full session
        [r.s2_pred,r.s2_err,r.s2_stat] = bci_predict(r.s1_cla,bci_preproc(eeg{2},paradigm,'events',full_session(i,j),options{:}), 'process',0);

        % train on first half of session 1
        %[r.h1_err,r.h1_cla,x,r.h1_stat] = bci_train({'func',paradigm,'data',eeg{1}},'events',half_session(i,j,'1'),options{:});        
        % test on second half of session 1
        %[r.h2_pred,r.h2_err,r.h2_stat] = bci_predict(r.h1_cla,bci_preproc(eeg{1},paradigm,'events',half_session(i,j,'2'),options{:}), 'process',0);
        
        r
        results{i,j} = r; 
    end
end