function res = process_concerto_icabci(varargin)
opts = hlp_varargin2struct(varargin,'paradigm','ultrawidecsp','comparison',{3,4},'online',1);
global r results; results = [];

% marker types
events = {{{'A_h1','A_h2'},{'B_h1','B_h2'},{'C_h1','C_h2'},{'F_h1','F_h2'},{'H_h1','H_h2'}}, {{'A_h1','A_h2'},{'B_h1','B_h2'},{'C_h1','C_h2'},{'F_h1','F_h2'},{'H_h1','H_h2'}}};

% turn on memory & disk cache, turn off unnecessary checks
exp_eval(Set(@memoize,{'memory',0,'disk',1}));



% select paradigm options
switch opts.paradigm
    case 'ultrawidecsp'
        paradigm = @para_csp; options = {'srate',[], 'channels',1:128, 'epoch',[-1 1], 'spectrum',[8 55; 65 200], opts}; 
    case 'ultrawidespeccsp'
        flt = @(f) f>8 & f<150 & ~(f>55 & f<65);
        paradigm = @para_speccsp; options = {'srate',300, 'channels',1:128, 'epoch',[-1 1], 'spectrum',[8 150], 'prior',flt, opts}; 
    case 'speccsp'
        flt = @(f) f>8 & f<55;
        paradigm = @para_speccsp; options = {'srate',150, 'channels',1:128, 'epoch',[-1 1], 'spectrum',[8 60], 'prior',flt, opts}; 
    case 'speccsplr'
        flt = @(f) f>8 & f<55;
        paradigm = @para_speccsp; options = {'srate',150, 'channels',1:128, 'epoch',[-1 1], 'spectrum',[8 60], 'prior',flt, 'learner',{'logreg',1,'variant','l2'},opts}; 
    case 'ultrawidespeccsplr'
        flt = @(f) f>8 & f<150 & ~(f>55 & f<65);
        paradigm = @para_speccsp; options = {'srate',300, 'channels',1:128, 'epoch',[-1 1], 'spectrum',[8 150], 'prior',flt, 'learner',{'logreg',1,'variant','l2'}, opts}; 
    case 'csplr'
        paradigm = @para_csp; options = {'srate',[], 'channels',1:128, 'epoch',[-1 1], 'spectrum',[7 55], 'learner',{'logreg',1,'variant','l2'}, opts}; 
    case 'dal'
        paradigm = @para_dal_hf; options = {'srate',150, 'channels',1:128, 'epoch',[-1 1], 'spectrum',[7 55],opts}; 
    otherwise
        error('unknown paradigm');
end

% train
session4ica = io_loadset('data:/concerto/tim-session4/ic-decomposed.set');
[r.trainerr,r.traincla,r.trainstats] = bci_train({'func',paradigm,'data',session4ica,'eval_scheme',0},'events',events{1},options{:});

% test 
io_load data:/concerto/tim-session3/raw.mat
[r.preds,r.testerr,r.teststats] = bci_predict(r.traincla,session3raw);



% plot time course
figure('position',[0 0 1600 300]);imagesc(filtfilt(ones(1,10),1,r.preds{2})');
title('Trained on session 4, tested on session 3');
ylabel('Predicted Distribution');
xlabel('Session time')
save2pdf('concerto-spec-4vs3.pdf',gcf)

% plot IC-weightings...
comps = {[1 2] [1 3] [1 4] [1 5] [2 3] [2 4] [2 5] [3 4] [3 5] [4 5]};
for c = 1:length(comps)
    figure; title([num2str(comps(1)) ' vs ' num2str(comps(2))]); vis_sizeplot(eeg,r.traincla.voting{comps(1),comps(2)});  end























%     
% % plot scalp maps
% figue('position',[0 0 1600 800])
% comps = {[1 2] [1 3] [1 4] [1 5] [2 3] [2 4] [2 5] [3 4] [3 5] [4 5]};
% for c = 1:length(comps)
%     for p = 1:6
%         subplot(6,length(comps),(p-1)*length(comps)+c);
%         topoplot(r.traincla.voting{comps{c}(1),comps{c}(2)}.featuremodel.W(:,p),r.traincla.bcilab.channel_locations); 
%         title(num2str(p));
%         camzoom(1.75);
%     end
% end
% 
% % plot spectral filters
% filtlen = 8;
% figure('position',[0 0 1600 800])
% comps = {[1 2] [1 3] [1 4] [1 5] [2 3] [2 4] [2 5] [3 4] [3 5] [4 5]};
% for c = 1:length(comps)
%     for p = 1:6
%         subplot(6,length(comps),(p-1)*length(comps)+c);
%         m = r.traincla.voting{comps{c}(1),comps{c}(2)}.featuremodel;
%         plot(m.freqs(m.bands),filtfilt(ones(1,filtlen),1,m.alpha(m.bands,p)));
%         %topoplot(r.traincla.voting{comps{c}(1),comps{c}(2)}.featuremodel.W(:,p),r.traincla.bcilab.channel_locations); 
%         title(num2str(p));
%     end
% end

%     % test the online system & display results
%     if opts.online
%         [r.predsom,r.latsom] = onl_stream(eeg{end},r.traincla,'markers',[events{end}{:}],'show_progress',1,'offset',512);
%         figure;imagesc(filtfilt(ones(1,100),1,r.predsom{2}(1:end,:))); title('online');
%         preds = r.predsom;
%     end


% save properly
classifier = r.traincla;
save emo-classifier-new.mat classifier

results = r;
res = r;