function vis_likelihoods(model,varargin)
o = hlp_varargin2struct(varargin, ...
    'smoothlen',1, ... % length of smoothing window in seconds or samples (latter if no srate is available)
    'eeg',[], ...      % eeg dataset
    'srate',1,...      % sampling rate
    'evtypes',{},...   % event-types of interest
    'wndfun',@hann...  % window function for smoothing
    );

% take srate from EEG, if available & not otherwise specified
if ~isempty(o.eeg) && o.srate == 1
	o.srate = o.eeg.srate; end

o.smoothlen = o.smoothlen*o.srate;
o.smoothwnd = o.wndfun(o.smoothlen);
o.smoothwnd = o.smoothwnd / sum(o.smoothwnd);

figure; hold;

% x coordinate
X = (0:length(model.v)-1) / o.srate;
% y coordinate(s)
Y = filtfilt_fast(o.smoothwnd,1,model.v');
% plot model time courses
plot(X,Y);
% add legend
legstrs = {};
for k=1:model.num_models
    legstrs{end+1} = ['model ' num2str(k)]; end
legend(legstrs{:});

% plot events
eeg = o.eeg;
if ~isempty(o.evtypes)
    for e=o.evtypes
        evt = e{1};
        lats = [eeg.event([eeg.event.type]==evt).latency];
        X=sort([lats lats-1 lats+1]) / o.srate * (length(m.v)/eeg.pnts);
        Y=repmat([0,1,0],1,length(lats));
        plot(X,Y,'--');
    end
end