function vis_sizeplot(dataset,model,chanlocs,zoomscale)
% vis_sizeplot(Dataset,Model,Zoomscale)
% displays the features used by the feature extraction part of CSP/SpecCSP classifiers when trained on IC-decomposed data
%
% In:
%   Dataset   : dataset that was used to train the classifier
%   
%   Model     : predictive model of the CSP/Spec-CSP family
%
%   Zoomscale : optional zoom factor (default: 2)
%
%                               Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                               2010-07-07

if ~exist('zoomscale','var') zoomscale = 2; end

% find chanlocs
if ~exist('chanlocs','var')
    try
        chanlocs = model.bcilab.channel_locations;
    catch
        try
            chanlocs = dataset.chanlocs;
        catch
            if isstruct(dataset)
                chanlocs = dataset;
            else
                error;
            end
        end
    end
end


if isnumeric(dataset)
    icawinv = dataset;
elseif isfield(dataset,'icawinv') && ~isempty(dataset.icawinv)
    icawinv = dataset.icawinv;
    chanlocs = chanlocs(dataset.icachansind);
else
    dataset = bci_preproc(dataset,model);
end

% get scaled weights
W = model.featuremodel.W .* repmat(abs(model.predictivemodel.model.w),size(model.featuremodel.W,1),1);
weights = sum(abs(W)');% ./ std(dataset.data,0,2)';

% number of subplots, per axis
N = size(weights,2);
py = ceil(sqrt(N));
px = ceil(N/py);

for k=1:N
    weight = weights(k)/max(weights);
    subplot(px,py,k); title(sprintf('%.1f',weight));
    topoplot(icawinv(:,k),chanlocs);
    camzoom(zoomscale*sqrt(weight));
end
